@@ -1,4 +1,9 @@
-;; Copyright (C) 1985, 1988, 1992 Free Software Foundation, Inc.
+;;; telnet.el --- run a telnet session from within an Emacs buffer
+
+;;; Copyright (C) 1985, 1988, 1992, 1993 Free Software Foundation, Inc.
+
+;; Author: William F. Schelter
+;; Maintainer: FSF
 
 ;; This file is part of GNU Emacs.
 
@@ -16,12 +21,27 @@
 ;; along with GNU Emacs; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
-;; Author William F. Schelter
-;; Hacked for lemacs and comint by Jamie Zawinski
+;;; Commentary:
 
-;;to do fix software types for lispm:
-;;to eval current expression.  Also to try to send escape keys correctly.
-;;essentially we'll want the rubout-handler off.
+;; This mode is intended to be used for telnet or rsh to a remode host;
+;; `telnet' and `rsh' are the two entry points.  Multiple telnet or rsh
+;; sessions are supported.
+;;
+;; Normally, input is sent to the remote telnet/rsh line-by-line, as you
+;; type RET or LFD.  C-c C-c sends a C-c to the remote immediately; 
+;; C-c C-z sends C-z immediately.  C-c C-q followed by any character
+;; sends that character immediately.
+;;
+;; All RET characters are filtered out of the output coming back from the
+;; remote system.  The mode tries to do other useful translations based
+;; on what it sees coming back from the other system before the password
+;; query.  It knows about UNIX, ITS, TOPS-20 and Explorer systems.
+
+;;; Code:
+
+;; to do fix software types for lispm:
+;; to eval current expression.  Also to try to send escape keys correctly.
+;; essentially we'll want the rubout-handler off.
 
 ;; filter is simplistic but should be okay for typical shell usage.
 ;; needs hacking if it is going to deal with asynchronous output in a sane
@@ -30,36 +50,29 @@
 (require 'comint)
 
 (defvar telnet-mode-map nil)
-
-(if telnet-mode-map
-    nil
-  (setq telnet-mode-map (make-sparse-keymap))
-  (set-keymap-name telnet-mode-map 'telnet-mode-map)
-  (set-keymap-parent telnet-mode-map comint-mode-map)
-  (define-key telnet-mode-map "\C-m" 'telnet-send-input)
-;  (define-key telnet-mode-map "\C-j" 'telnet-send-input)
-  (define-key telnet-mode-map "\C-c\C-q" 'telnet-send-process-next-char)
-  (define-key telnet-mode-map "\C-c\C-c" 'telnet-interrupt-subjob) 
-  (define-key telnet-mode-map "\C-c\C-z" 'telnet-c-z))
-
-(defvar telnet-new-line "\r")				; buffer-local
-(defvar telnet-replace-c-g nil)				; buffer-local
-(defvar telnet-prompt-pattern "^[^#$%>]*[#$%>] *")	; buffer-local
-(defvar telnet-remote-echoes t				; buffer-local
+(defvar telnet-new-line "\r")
+(make-variable-buffer-local 'telnet-new-line)
+(defvar telnet-prompt-pattern "^[^#$%>]*[#$%>] *")
+(defvar telnet-replace-c-g nil)
+(make-variable-buffer-local 'telnet-replace-c-g)
+(defvar telnet-remote-echoes t
    "True if the telnet process will echo input.")
-(defvar telnet-interrupt-string "\C-c"			; buffer-local
+(make-variable-buffer-local 'telnet-remote-echoes)
+(defvar telnet-interrupt-string "\C-c"
   "String sent by C-c.")
+(make-variable-buffer-local 'telnet-interrupt-string)
 
-(defvar telnet-count 0					; buffer-local
+(defvar telnet-count 0
   "Number of output strings read from the telnet process
 while looking for the initial password.")
+(make-variable-buffer-local 'telnet-count)
 
 (defvar telnet-initial-count -50
-  "Initial value of telnet-count.  Should be set to the negative of the
+  "Initial value of `telnet-count'.  Should be set to the negative of the
 number of terminal writes telnet will make setting up the host connection.")
 
 (defvar telnet-maximum-count 4
-  "Maximum value telnet-count can have.
+  "Maximum value `telnet-count' can have.
 After this many passes, we stop looking for initial setup data.
 Should be set to the number of terminal writes telnet will make
 rejecting one login and prompting for the again for a username and password.")
@@ -67,36 +80,47 @@ rejecting one login and prompting for the again for a username and password.")
 (defun telnet-interrupt-subjob ()
   (interactive)
   "Interrupt the program running through telnet on the remote host."
-  (send-string nil telnet-interrupt-string))
+  (process-send-string nil telnet-interrupt-string))
 
 (defun telnet-c-z ()
   (interactive)
-  (send-string nil "\C-z"))
+  (process-send-string nil "\C-z"))
 
 (defun telnet-send-process-next-char ()
   (interactive)
-  (send-string nil
+  (process-send-string nil
 	       (char-to-string
 		(let ((inhibit-quit t))
 		  (prog1 (read-char)
 		    (setq quit-flag nil))))))
 
+; initialization on first load.
+(if telnet-mode-map
+    nil
+  (progn
+    (setq telnet-mode-map (make-sparse-keymap))
+    (set-keymap-name telnet-mode-map 'telnet-mode-map)
+    (set-keymap-parent telnet-mode-map comint-mode-map)
+    (define-key telnet-mode-map "\C-m" 'telnet-send-input)
+    ;;(define-key telnet-mode-map "\C-j" 'telnet-send-input)
+    (define-key telnet-mode-map "\C-c\C-q" 'telnet-send-process-next-char)
+    (define-key telnet-mode-map "\C-c\C-c" 'telnet-interrupt-subjob) 
+    (define-key telnet-mode-map "\C-c\C-z" 'telnet-c-z)))
+
 ;;maybe should have a flag for when have found type
 (defun telnet-check-software-type-initialize (string)
   "Tries to put correct initializations in.  Needs work."
   (let ((case-fold-search t))
     (cond ((string-match "unix" string)
-	   ;;(setq telnet-prompt-pattern comint-prompt-regexp)
-	   (setq telnet-prompt-pattern shell-prompt-pattern)
+	   (setq telnet-prompt-pattern comint-prompt-regexp)
 	   (setq telnet-new-line "\n"))
-	  ((string-match "tops-20" string);;maybe add telnet-replace-c-g
+	  ((string-match "tops-20" string) ;;maybe add telnet-replace-c-g
 	   (setq telnet-prompt-pattern  "[@>]*"))
 	  ((string-match "its" string)
 	   (setq telnet-prompt-pattern  "^[^*>]*[*>] *"))
-	  ((string-match "explorer" string);;explorer telnet needs work
+	  ((string-match "explorer" string) ;;explorer telnet needs work
 	   (setq telnet-replace-c-g ?\n))))
-  (set (make-local-variable 'comint-prompt-regexp)
-       telnet-prompt-pattern))
+  (setq comint-prompt-regexp telnet-prompt-pattern))
 
 (defun telnet-initial-filter (proc string)
   ;For reading up to and including password; also will get machine type.
@@ -105,16 +129,21 @@ rejecting one login and prompting for the again for a username and password.")
 	 (error "No such host."))
 	((string-match "passw" string)
 	 (telnet-filter proc string)
-	 (let* ((echo-keystrokes 0)
-		(password (telnet-read-password)))
+	 (let ((password (comint-read-noecho "Password: ")))
 	   (setq telnet-count 0)
-	   (send-string proc (concat password telnet-new-line))))
+	   (process-send-string proc (concat password telnet-new-line))))
 	(t (telnet-check-software-type-initialize string)
 	   (telnet-filter proc string)
 	   (cond ((> telnet-count telnet-maximum-count)
 		  (set-process-filter proc 'telnet-filter))
 		 (t (setq telnet-count (1+ telnet-count)))))))
 
+;; Identical to comint-simple-send, except that it sends telnet-new-line
+;; instead of "\n".
+(defun telnet-simple-send (proc string)
+  (comint-send-string proc string)
+  (comint-send-string proc telnet-new-line))
+
 (defun telnet-filter (proc string)
   (save-excursion
     (set-buffer (process-buffer proc))
@@ -126,6 +155,7 @@ rejecting one login and prompting for the again for a username and password.")
 	    (w (get-buffer-window (current-buffer)))
 	    (ws (and w (window-start w))))
        (goto-char last-insertion)
+	;; Insert STRING, omitting all C-m characters.
        (insert-before-markers string)
        (set-marker (process-mark proc) (point))
        ;; the insert-before-markers may have screwed window-start
@@ -145,15 +175,6 @@ rejecting one login and prompting for the again for a username and password.")
 	   (shell-hack-prompt-font last-insertion))
        ))))
 
-(defun telnet-read-password ()
-  (let ((answ "") tem)
-    (message "Reading password...")
-    (while (not (or (= (setq tem (read-char)) ?\^m)
-		    (= tem ?\n)))
-      (setq answ (concat answ (char-to-string tem))))
-    (message "")
-    answ))
-
 (defun telnet-send-input ()
   (interactive)
   (let ((proc (get-buffer-process (current-buffer)))
@@ -173,32 +194,56 @@ rejecting one login and prompting for the again for a username and password.")
       (if p1
 	  (delete-region p1 (1+ p2))))))
 
-(defun telnet-input-sender (proc string)
-  (comint-send-string proc string)
-  (comint-send-string proc telnet-new-line))
-
-(defun telnet (hostname)
+;;;###autoload
+(defun telnet (host)
   "Open a network login connection to host named HOST (a string).
 Communication with HOST is recorded in a buffer *HOST-telnet*.
 Normally input is edited in Emacs and sent a line at a time.
 See also `\\[rsh]'."
   (interactive "sOpen telnet connection to host: ")
-  (let ((name (concat hostname "-telnet")))
-    (switch-to-buffer (make-comint name "telnet"))
-    (set-process-filter (get-process name) 'telnet-initial-filter)
-    (erase-buffer)
-    (send-string  name (concat "open " hostname "\n"))
-    (telnet-mode)
-    (setq telnet-count telnet-initial-count)))
+  (let* ((name (concat host "-telnet"))
+         (buffer (get-buffer (concat "*" name "*"))))
+    (if (and buffer (get-buffer-process buffer))
+	(switch-to-buffer buffer)
+      (progn
+	(switch-to-buffer (make-comint name "telnet"))
+	(set-process-filter (get-process name) 'telnet-initial-filter)
+	;; Don't send the `open' cmd till telnet is ready for it.
+	(accept-process-output (get-process name))
+	(erase-buffer)
+	(process-send-string name (concat "open " host "\n"))
+	(telnet-mode)
+	(setq comint-input-sender 'telnet-simple-send)
+	(setq telnet-count telnet-initial-count)))))
 
-(defun rsh (hostname)
+(defun telnet-mode ()
+  "This mode is for using telnet (or rsh) from a buffer to another host.
+It has most of the same commands as comint-mode.
+There is a variable ``telnet-interrupt-string'' which is the character
+sent to try to stop execution of a job on the remote host.
+Data is sent to the remote host when RET is typed.
+
+\\{telnet-mode-map}
+"
+  (interactive)
+  (comint-mode)
+  (setq major-mode 'telnet-mode
+        mode-name "Telnet"
+        comint-prompt-regexp telnet-prompt-pattern)
+  (use-local-map telnet-mode-map)
+  (run-hooks 'telnet-mode-hook))
+
+;; Berkeley spawn of hell
+;;;###autoload
+(defun rsh (host)
   "Open a network login connection to host named HOST (a string).
 Communication with HOST is recorded in a buffer *HOST-rsh*.
 Normally input is edited in Emacs and sent a line at a time.
 See also `\\[telnet]'."
   (interactive "sOpen rsh connection to host: ")
-  (let ((name (concat hostname "-rsh")))
-    (switch-to-buffer (make-comint name "rsh" nil hostname))
+  (require 'shell)
+  (let ((name (concat host "-rsh")))
+    (switch-to-buffer (make-comint name "rsh" nil host))
     (set-process-filter (get-process name) 'telnet-initial-filter)
     (telnet-mode)
     ;; SunOS doesn't print "unix" in its rsh login banner, so let's get a
@@ -208,29 +253,4 @@ See also `\\[telnet]'."
     (telnet-check-software-type-initialize "unix")
     (setq telnet-count telnet-initial-count)))
 
-(defun telnet-mode ()
-  "This mode is for use during telnet or rsh from a buffer to another
-host. It has most of the same commands as comint-mode.
-There is a variable ``telnet-interrupt-string'' which is the character
-sent to try to stop execution of a job on the remote host.
-Data is sent to the remote host when RET is typed.
-\\{telnet-mode-map}
-
-Bugs:
---Replaces \C-m by a space, really should remove."
-  (interactive)
-  (comint-mode)					    ; runs comint-mode-hook
-  (setq major-mode 'telnet-mode)
-  (setq mode-name "Telnet")
-  (setq comint-prompt-regexp telnet-prompt-pattern) ; local via comint-mode
-  (setq comint-input-sender 'telnet-input-sender)   ; local via comint-mode
-  (make-local-variable 'telnet-prompt-pattern)
-  (make-local-variable 'telnet-new-line)
-  (make-local-variable 'telnet-replace-c-g)
-  (make-local-variable 'telnet-remote-echoes)
-  (make-local-variable 'telnet-interrupt-string)
-  (make-local-variable 'telnet-count)
-  (use-local-map telnet-mode-map)
-  (run-hooks 'telnet-mode-hook))
-
 (provide 'telnet)
