@@ -58,7 +58,8 @@ This should be a fractional part of a second (a float.)")
   (let ((dir (blink-paren-sexp-dir)))
     (and dir
 	 (condition-case ()
-	     (let* ((other-pos (save-excursion (forward-sexp dir) (point)))
+	     (let* ((parse-sexp-ignore-comments t)
+		    (other-pos (save-excursion (forward-sexp dir) (point)))
 		    (extent (if (= dir 1)
 				(make-extent (- other-pos 1) other-pos)
 			      (make-extent other-pos (+ other-pos 1)))))
