@@ -1,5 +1,6 @@
-;; Replace commands for Emacs.
-;; Copyright (C) 1985-1993 Free Software Foundation, Inc.
+;;; replace.el --- replace commands for Emacs.
+
+;; Copyright (C) 1985, 1986, 1987, 1992, 1993 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -17,17 +18,144 @@
 ;; along with GNU Emacs; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
+;;; Commentary:
 
-(defvar minibuffer-regexp-history nil)
+;; This package supplies the string and regular-expression replace functions
+;; documented in the Emacs user's manual.
 
-(fset 'delete-non-matching-lines 'keep-lines)
-(defun keep-lines (regexp)
+;;; Code:
+
+;; in loaddefs
+;(defvar case-replace t "\
+;*Non-nil means query-replace should preserve case in replacements.")
+
+(defvar query-replace-history nil)
+
+(defun query-replace-read-args (string)
+  (let (from to)
+    (setq from (read-from-minibuffer (format "%s: " string)
+				     nil nil nil
+				     'query-replace-history))
+    (setq to (read-from-minibuffer (format "%s %s with: " string from)
+				   nil nil nil
+				   'query-replace-history))
+    (list from to current-prefix-arg)))
+
+(defun query-replace (from-string to-string &optional arg)
+  "Replace some occurrences of FROM-STRING with TO-STRING.
+As each match is found, the user must type a character saying
+what to do with it.  For directions, type \\[help-command] at that time.
+
+Preserves case in each replacement if  case-replace  and  case-fold-search
+are non-nil and FROM-STRING has no uppercase letters.
+Third arg DELIMITED (prefix arg if interactive) non-nil means replace
+only matches surrounded by word boundaries.
+
+To customize possible responses, change the \"bindings\" in `query-replace-map'."
+  (interactive (query-replace-read-args "Query replace"))
+  (perform-replace from-string to-string t nil arg)
+  (or unread-command-event (message "Done")))
+
+(defun query-replace-regexp (regexp to-string &optional arg)
+  "Replace some things after point matching REGEXP with TO-STRING.
+As each match is found, the user must type a character saying
+what to do with it.  For directions, type \\[help-command] at that time.
+
+Preserves case in each replacement if  case-replace  and  case-fold-search
+are non-nil and REGEXP has no uppercase letters.
+Third arg DELIMITED (prefix arg if interactive) non-nil means replace
+only matches surrounded by word boundaries.
+In TO-STRING, \\& means insert what matched REGEXP,
+and \\=\\<n> means insert what matched <n>th \\(...\\) in REGEXP."
+  (interactive (query-replace-read-args "Query replace regexp"))
+  (perform-replace regexp to-string t t arg)
+  (or unread-command-event (message "Done")))
+
+;;>>> Not patently useful
+(defun map-query-replace-regexp (regexp to-strings &optional arg)
+  "Replace some matches for REGEXP with various strings, in rotation.
+The second argument TO-STRINGS contains the replacement strings, separated
+by spaces.  This command works like `query-replace-regexp' except
+that each successive replacement uses the next successive replacement string,
+wrapping around from the last such string to the first.
+
+Non-interactively, TO-STRINGS may be a list of replacement strings.
+
+A prefix argument N says to use each replacement string N times
+before rotating to the next."
+  (interactive
+   (let (from to)
+     (setq from (read-from-minibuffer "Map query replace (regexp): "
+				      nil nil nil
+				      'query-replace-history))
+     (setq to (read-from-minibuffer
+	       (format "Query replace %s with (space-separated strings): "
+		       from)
+	       nil nil nil
+	       'query-replace-history))
+     (list from to current-prefix-arg)))
+  (let (replacements)
+    (if (listp to-strings)
+	(setq replacements to-strings)
+      (while (/= (length to-strings) 0)
+	(if (string-match " " to-strings)
+	    (setq replacements
+		  (append replacements
+			  (list (substring to-strings 0
+					   (string-match " " to-strings))))
+		  to-strings (substring to-strings
+				       (1+ (string-match " " to-strings))))
+	  (setq replacements (append replacements (list to-strings))
+		to-strings ""))))
+   (perform-replace regexp replacements t t nil arg))
+  (or unread-command-event (message "Done")))
+
+(defun replace-string (from-string to-string &optional delimited)
+  "Replace occurrences of FROM-STRING with TO-STRING.
+Preserve case in each match if `case-replace' and `case-fold-search'
+are non-nil and FROM-STRING has no uppercase letters.
+Third arg DELIMITED (prefix arg if interactive) non-nil means replace
+only matches surrounded by word boundaries.
+
+This function is usually the wrong thing to use in a Lisp program.
+What you probably want is a loop like this:
+  (while (search-forward OLD-STRING nil t)
+    (replace-match REPLACEMENT nil t))
+which will run faster and will not set the mark or print anything."
+  (interactive (query-replace-read-args "Replace string"))
+  (perform-replace from-string to-string nil nil delimited)
+  (or unread-command-event (message "Done")))
+
+(defun replace-regexp (regexp to-string &optional delimited)
+  "Replace things after point matching REGEXP with TO-STRING.
+Preserve case in each match if case-replace and case-fold-search
+are non-nil and REGEXP has no uppercase letters.
+Third arg DELIMITED (prefix arg if interactive) non-nil means replace
+only matches surrounded by word boundaries.
+In TO-STRING, \\& means insert what matched REGEXP,
+and \\=\\<n> means insert what matched <n>th \\(...\\) in REGEXP.
+
+This function is usually the wrong thing to use in a Lisp program.
+What you probably want is a loop like this:
+  (while (re-search-forward REGEXP nil t)
+    (replace-match REPLACEMENT nil nil))
+which will run faster and will not set the mark or print anything."
+  (interactive (query-replace-read-args "Replace regexp"))
+  (perform-replace regexp to-string nil t delimited)
+  (or unread-command-event (message "Done")))
+
+
+(defvar regexp-history nil
+  "History list for some commands that read regular expressions.")
+
+(define-function 'keep-lines 'delete-non-matching-lines)
+(defun delete-non-matching-lines (regexp)
   "Delete all lines except those containing matches for REGEXP.
 A match split across lines preserves all the lines it lies in.
 Applies to all lines after point."
-;;  (interactive "sKeep lines (containing match for regexp): ")
-  (interactive (list (read-string "Keep lines (containing match for regexp): "
-				  nil 'minibuffer-regexp-history)))
+  (interactive (list (read-from-minibuffer
+		      "Keep lines (containing match for regexp): "
+		      nil nil nil 'regexp-history)))
   (save-excursion
     (or (bolp) (forward-line 1))
     (let ((start (point)))
@@ -47,14 +175,14 @@ Applies to all lines after point."
 	(and (not (eobp)) (= (match-beginning 0) (match-end 0))
 	     (forward-char 1))))))
 
-(fset 'delete-matching-lines 'flush-lines)
-(defun flush-lines (regexp)
+(define-function 'flush-lines 'delete-matching-lines)
+(defun delete-matching-lines (regexp)
   "Delete lines containing matches for REGEXP.
 If a match is split across lines, all the lines it lies in are deleted.
 Applies to lines after point."
-;;  (interactive "sFlush lines (containing match for regexp): ")
-  (interactive (list (read-string "Flush lines (containing match for regexp): "
-				  nil 'minibuffer-regexp-history)))
+  (interactive (list (read-from-minibuffer
+		      "Flush lines (containing match for regexp): "
+		      nil nil nil 'regexp-history)))
   (save-excursion
     (while (and (not (eobp))
 		(re-search-forward regexp nil t))
@@ -63,12 +191,12 @@ Applies to lines after point."
 				     (point))
 		     (progn (forward-line 1) (point))))))
 
-(fset 'count-matches 'how-many)
-(defun how-many (regexp)
+(define-function 'how-many 'count-matches)
+(defun count-matches (regexp)
   "Print number of matches for REGEXP following point."
-;;  (interactive "sHow many matches for (regexp): ")
-  (interactive (list (read-string "How many matches for (regexp): "
-			    nil 'minibuffer-regexp-history)))
+  (interactive (list (read-from-minibuffer
+		      "How many matches for (regexp): "
+		      nil nil nil 'regexp-history)))
   (let ((count 0) opoint)
     (save-excursion
      (while (and (not (eobp))
@@ -79,16 +207,18 @@ Applies to lines after point."
 	 (setq count (1+ count))))
      (message "%d occurrences" count))))
 
+
 (defvar occur-mode-map ())
 (if occur-mode-map
     ()
   (setq occur-mode-map (make-sparse-keymap))
-  (define-key occur-mode-map "\C-c\C-c" 'occur-mode-goto-occurrence))
+  (set-keymap-name occur-mode-map 'occur-mode-map)
+  (define-key occur-mode-map "\C-c\C-c" 'occur-mode-goto-occurrence)
+  (define-key occur-mode-map 'button2 'occur-mouse-goto))
 
 (defvar occur-buffer nil)
 (defvar occur-nlines nil)
 (defvar occur-pos-list nil)
-(defvar occur-last-string "")
 
 (defun occur-mode ()
   "Major mode for output from \\[occur].
@@ -102,7 +232,14 @@ in the buffer that the occurrences were found in.
   (setq mode-name "Occur")
   (make-local-variable 'occur-buffer)
   (make-local-variable 'occur-nlines)
-  (make-local-variable 'occur-pos-list))
+  (make-local-variable 'occur-pos-list)
+  (require 'mode-motion)
+  (setq mode-motion-hook 'mode-motion-highlight-line))
+
+(defun occur-mouse-goto (e)
+  (interactive "e")
+  (mouse-set-point e)
+  (occur-mode-goto-occurrence))
 
 (defun occur-mode-goto-occurrence ()
   "Go to the line this occurrence was found in, in the buffer it was found in."
@@ -125,45 +262,58 @@ in the buffer that the occurrences were found in.
 				   (+ 2 (* 2 occur-nlines)))
 				  (t 1)))))
 	 (pos (nth occur-number occur-pos-list)))
+    (or pos
+	(error "No occurrence on this line"))
     (pop-to-buffer occur-buffer)
     (goto-char (marker-position pos))))
 
+
 (defvar list-matching-lines-default-context-lines 0
-  "*Default number of context lines to include around a list-matching-lines
+  "*Default number of context lines to include around a `list-matching-lines'
 match.  A negative number means to include that many lines before the match.
 A positive number means to include that many lines both before and after.")
 
+;;>>> Should be named list-matching-lines-whole-buffer
 (defvar occur-whole-buffer nil
   "If t, occur operates on whole buffer, otherwise occur starts from point.
 default is nil.")
 
-(fset 'list-matching-lines 'occur)
+(define-function 'occur 'list-matching-lines)
+(defun list-matching-lines (regexp &optional nlines)
+  "Show all lines in the current buffer containing a match for REGEXP.
 
-(defun occur (regexp &optional nlines)
-  "Show lines containing a match for REGEXP.  If the global variable
-occur-whole-buffer is non-nil, the entire buffer is searched, otherwise
-search begins at point.  Interactively, REGEXP defaults to the last REGEXP
-used interactively.
+If a match spreads across multiple lines, all those lines are shown.
+
+If variable `occur-whole-buffer' is non-nil, the entire buffer is
+searched, otherwise search begins at point.
 
 Each line is displayed with NLINES lines before and after,
 or -NLINES before if NLINES is negative.
-NLINES defaults to list-matching-lines-default-context-lines.
+NLINES defaults to `list-matching-lines-default-context-lines'.
 Interactively it is the prefix arg.
 
-The lines are shown in a buffer named *Occur*.
+The lines are shown in a buffer named `*Occur*'.
 It serves as a menu to find any of the occurrences in this buffer.
 \\[describe-mode] in that buffer will explain how."
-  (interactive (list (setq occur-last-string
-			   (read-string "List lines matching regexp: "
-					occur-last-string
-					'minibuffer-regexp-history))
+  (interactive (list (let* ((default (car regexp-history))
+			    (input 
+			     (read-from-minibuffer
+			      (if default
+				  (format "List lines matching regexp (default `%s'): " default)
+                                  "List lines matching regexp: ")
+			      nil nil nil
+			      'regexp-history)))
+		       (if (> (length input) 0) 
+                           input
+                           (setcar regexp-history default)))
 		     current-prefix-arg))
   (setq nlines (if nlines (prefix-numeric-value nlines)
 		 list-matching-lines-default-context-lines))
   (let ((first t)
 	(buffer (current-buffer))
 	(linenum 1)
-	(prevpos (point-min)))
+	(prevpos (point-min))
+        (final-context-start (make-marker)))
     (if (not occur-whole-buffer)
 	(save-excursion
 	  (beginning-of-line)
@@ -187,13 +337,18 @@ It serves as a menu to find any of the occurrences in this buffer.
 	;; Find next match, but give up if prev match was at end of buffer.
 	(while (and (not (= prevpos (point-max)))
 		    (re-search-forward regexp nil t))
+	  (goto-char (match-beginning 0))
 	  (beginning-of-line)
-	  (setq linenum (+ linenum (count-lines prevpos (point))))
+	  (save-match-data
+            (setq linenum (+ linenum (count-lines prevpos (point)))))
 	  (setq prevpos (point))
+	  (goto-char (match-end 0))
 	  (let* ((start (save-excursion
+			  (goto-char (match-beginning 0))
 			  (forward-line (if (< nlines 0) nlines (- nlines)))
 			  (point)))
 		 (end (save-excursion
+			(goto-char (match-end 0))
 			(if (> nlines 0)
 			    (forward-line (1+ nlines))
 			    (forward-line 1))
@@ -216,12 +371,20 @@ It serves as a menu to find any of the occurrences in this buffer.
 		(insert empty ?:)
 		(forward-line 1)
 		(setq tem (1- tem)))
-	      (insert tag ?:)
-	      (forward-line 1)
+	      (let ((this-linenum linenum))
+		(set-marker final-context-start
+			    (+ (point) (- (match-end 0) (match-beginning 0))))
+		(while (< (point) final-context-start)
+		  (if (null tag)
+		      (setq tag (format "%3d" this-linenum)))
+		  (insert tag ?:)
+		  (setq tag nil)
+		  (forward-line 1)
+		  (setq this-linenum (1+ this-linenum))))
 	      (while (< tem nlines)
 		(insert empty ?:)
 		(forward-line 1)
-		(setq tem (1+ tem))))				
+		(setq tem (1+ tem))))
 	    (forward-line 1)))
 	(set-buffer standard-output)
 	;; Put positions in increasing order to go with buffer.
@@ -229,7 +392,9 @@ It serves as a menu to find any of the occurrences in this buffer.
 	(if (interactive-p)
 	    (message "%d matching lines." (length occur-pos-list)))))))
 
-(defconst query-replace-help
+;; It would be nice to use \\[...], but there is no reasonable way
+;; to make that display both SPC and Y.
+(defvar query-replace-help (purecopy
   "Type Space or `y' to replace one match, Delete or `n' to skip to next,
 ESC or `q' to exit, Period to replace one match and exit,
 Comma to replace but not move point immediately,
@@ -237,9 +402,44 @@ C-r to enter recursive edit (\\[exit-recursive-edit] to get out again),
 C-w to delete match and recursive edit,
 C-l to clear the screen, redisplay, and offer same replacement again,
 ! to replace all remaining matches with no more questions,
-^ to move point back to previous match."
+^ to move point back to previous match.")
   "Help message while in query-replace")
 
+(defvar	query-replace-map nil
+  "Keymap that defines the responses to questions in `query-replace'.
+The \"bindings\" in this map are not commands; they are answers.
+The valid answers include `act', `skip', `act-and-show',
+`exit', `act-and-exit', `edit', `delete-and-edit', `recenter',
+`automatic', `backup', and `help'.")
+
+(if query-replace-map
+    nil
+    (let ((map (make-sparse-keymap)))
+      (set-keymap-name map 'query-replace-map)
+      (define-key map " " 'act)
+      (define-key map "\d" 'skip)
+      (define-key map [delete] 'skip)
+      (define-key map [backspace] 'skip)
+      (define-key map "y" 'act)
+      (define-key map "n" 'skip)
+      (define-key map "," 'act-and-show)
+      (define-key map [escape] 'exit)
+      (define-key map "q" 'exit)
+      (define-key map [return] 'exit)
+      (define-key map "." 'act-and-exit)
+      (define-key map "\C-r" 'edit)
+      (define-key map "\C-w" 'delete-and-edit)
+      (define-key map "\C-l" 'recenter)
+      (define-key map "!" 'automatic)
+      (define-key map "^" 'backup)
+      (define-key map "\C-h" 'help)
+      (define-key map "?" 'help)
+      (define-key map "\C-g" 'quit)
+      (define-key map "\C-]" 'quit)
+      
+      (setq query-replace-map map)))
+
+
 (autoload 'isearch-highlight "isearch")
 
 (defun perform-replace-next-event (event)
@@ -255,28 +455,30 @@ C-l to clear the screen, redisplay, and offer same replacement again,
 
 (defun perform-replace (from-string replacements
 		        query-flag regexp-flag delimited-flag
-			&optional repeat-count)
+			&optional repeat-count map)
   "Subroutine of `query-replace'.  Its complexity handles interactive queries.
 Don't use this in your own program unless you want to query and set the mark
 just as `query-replace' does.  Instead, write a simple loop like this:
   (while (re-search-forward \"foo[ \t]+bar\" nil t)
     (replace-match \"foobar\" nil nil))
 which will run faster and do exactly what you probably want."
-  (let ((char nil)
-	(event (allocate-event))
+  (or map (setq map query-replace-map))
+  (let ((event (allocate-event))
 	(nocasify (not (and case-fold-search case-replace
 			    (string-equal from-string
 					  (downcase from-string)))))
 	(literal (not regexp-flag))
 	(search-function (if regexp-flag 're-search-forward 'search-forward))
 	(search-string from-string)
+	(real-match-data nil)		; the match data for the current match
 	(next-replacement nil)
 	(replacement-index 0)
 	(keep-going t)
 	(stack nil)
 	(next-rotate-count 0)
 	(replace-count 0)
-	(lastrepl nil))			;Position after last match considered.
+	(lastrepl nil)			;Position after last match considered.
+	(match-again t))
     (if (stringp replacements)
 	(setq next-replacement replacements)
       (or repeat-count (setq repeat-count 1)))
@@ -288,16 +490,33 @@ which will run faster and do exactly what you probably want."
 				    "\\b")))
     (push-mark)
     (undo-boundary)
+    ;; Loop finding occurrences that perhaps should be replaced.
     (while (and keep-going
 		(not (eobp))
 		(funcall search-function search-string nil t)
-		(if (eq lastrepl (point))
-		    (progn 
+		;; If the search string matches immediately after
+		;; the previous match, but it did not match there
+		;; before the replacement was done, ignore the match.
+		(if (or (eq lastrepl (point))
+			(and regexp-flag
+			     (eq lastrepl (match-beginning 0))
+			     (not match-again)))
+
+		    (if (eobp)
+			nil
 		      ;; Don't replace the null string 
 		      ;; right after end of previous replacement.
 		      (forward-char 1)
 		      (funcall search-function search-string nil t))
 		  t))
+      ;; Save the data associated with the real match.
+      (setq real-match-data (match-data))
+
+      ;; Before we make the replacement, decide whether the search string
+      ;; can match again just after this match.
+      (if regexp-flag
+	  (setq match-again (looking-at search-string)))
+
       ;; If time for a change, advance to next replacement string.
       (if (and (listp replacements)
 	       (= next-rotate-count replace-count))
@@ -308,73 +527,79 @@ which will run faster and do exactly what you probably want."
 	    (setq replacement-index (% (1+ replacement-index) (length replacements)))))
       (if (not query-flag)
 	  (progn
+	    (store-match-data real-match-data)
 	    (replace-match next-replacement nocasify literal)
 	    (setq replace-count (1+ replace-count)))
 	(undo-boundary)
-	(let (done replaced)
-	  (while (not done)
-	    ;; Preserve the match data.  Process filters and sentinels
-	    ;; could run inside read-char..
-	    (let ((data (match-data))
-		  (help-form
-		   '(concat "Query replacing "
+	(let ((help-form '(concat "Query replacing "
 			    (if regexp-flag "regexp " "")
 			    from-string " with " next-replacement ".\n\n"
-			    (substitute-command-keys query-replace-help))))
-	      (setq char help-char)
-	      (while (or (not (numberp char)) (= char help-char))
-		(message "Query replacing %s with %s: " from-string next-replacement)
-		(perform-replace-next-event event)
-		
-		(setq char (or (event-to-character event) event))
-		(if (and (numberp char) (= char ??))
-		    (setq unread-command-event
-			   (character-to-event help-char (allocate-event))
-			  char help-char)))
-	      (store-match-data data))
-	    (cond ((or (= char ?\e)
-		       (= char ?q))
+                           (substitute-command-keys query-replace-help)))
+              (done nil)
+              (replaced nil)
+              def)
+          ;; Loop reading commands until one of them sets done,
+	  ;; which means it has finished handling this occurrence.
+	  (while (not done)
+            (message "Query replacing %s with %s: "
+                     from-string next-replacement)
+            (perform-replace-next-event event)
+            (setq def (lookup-key map (vector event)))
+	    ;; Restore the match data while we process the command.
+            (store-match-data real-match-data)
+	    (cond ((eq def 'help)
+		   (with-output-to-temp-buffer "*Help*"
+		     (princ
+		      (concat "Query replacing "
+			      (if regexp-flag "regexp " "")
+			      from-string " with "
+			      next-replacement ".\n\n"
+			      (substitute-command-keys
+			       query-replace-help)))))
+		  ((eq def 'exit)
 		   (setq keep-going nil)
 		   (setq done t))
-		  ((= char ?^)
+		  ((eq def 'backup)
 		   (let ((elt (car stack)))
 		     (goto-char (car elt))
 		     (setq replaced (eq t (cdr elt)))
 		     (or replaced
 			 (store-match-data (cdr elt)))
 		     (setq stack (cdr stack))))		     
-		  ((or (= char ?\ )
-		       (= char ?y))
+		  ((eq def 'act)
 		   (or replaced
 		       (replace-match next-replacement nocasify literal))
 		   (setq done t replaced t))
-		  ((= char ?\.)
+		  ((eq def 'act-and-exit)
 		   (or replaced
 		       (replace-match next-replacement nocasify literal))
 		   (setq keep-going nil)
 		   (setq done t replaced t))
-		  ((= char ?\,)
+		  ((eq def 'act-and-show)
 		   (if (not replaced)
 		       (progn
 			 (replace-match next-replacement nocasify literal)
 			 (setq replaced t))))
-		  ((= char ?!)
+		  ((eq def 'automatic)
 		   (or replaced
 		       (replace-match next-replacement nocasify literal))
 		   (setq done t query-flag nil replaced t))
-		  ((or (= char ?\177)
-		       (= char ?n))
+		  ((eq def 'skip)
 		   (setq done t))
-		  ((= char ?\C-l)
+		  ((eq def 'recenter)
 		   (recenter nil))
-		  ((= char ?\C-r)
+		  ((eq def 'edit)
 		   (store-match-data
 		    (prog1 (match-data)
-		      (save-excursion (recursive-edit)))))
-		  ((= char ?\C-w)
+		      (save-excursion (recursive-edit))))
+		   ;; Before we make the replacement,
+		   ;; decide whether the search string
+		   ;; can match again just after this match.
+		   (if regexp-flag
+		       (setq match-again (looking-at search-string))))
+		  ((eq def 'delete-and-edit)
 		   (delete-region (match-beginning 0) (match-end 0))
-		   (store-match-data
-		    (prog1 (match-data)
+		   (store-match-data (prog1 (match-data)
 		      (save-excursion (recursive-edit))))
 		   (setq replaced t))
 		  (t
@@ -388,41 +613,14 @@ which will run faster and do exactly what you probably want."
 		(cons (cons (point)
 			    (or replaced
 				(mapcar
-				 (function (lambda (elt)
-					     (if (markerp elt)
-						 (marker-position elt)
-					       elt)))
+				 #'(lambda (elt)
+				     (if (markerp elt)
+					 (marker-position elt)
+				        elt))
 				 (match-data))))
 		      stack))
 	  (if replaced (setq replace-count (1+ replace-count)))))
       (setq lastrepl (point)))
     (and keep-going stack)))
-
-(defun map-query-replace-regexp (regexp to-strings &optional arg)
-  "Replace some matches for REGEXP with various strings, in rotation.
-The second argument TO-STRINGS contains the replacement strings, separated
-by spaces.  This command works like `query-replace-regexp' except
-that each successive replacement uses the next successive replacement string,
-wrapping around from the last such string to the first.
-
-Non-interactively, TO-STRINGS may be a list of replacement strings.
-
-A prefix argument N says to use each replacement string N times
-before rotating to the next."
-  (interactive "sMap query replace (regexp): \nsQuery replace %s with (space-separated strings): \nP")
-  (let (replacements)
-    (if (listp to-strings)
-	(setq replacements to-strings)
-      (while (/= (length to-strings) 0)
-	(if (string-match " " to-strings)
-	    (setq replacements
-		  (append replacements
-			  (list (substring to-strings 0
-					   (string-match " " to-strings))))
-		  to-strings (substring to-strings
-				       (1+ (string-match " " to-strings))))
-	  (setq replacements (append replacements (list to-strings))
-		to-strings ""))))
-    (perform-replace regexp replacements t t nil arg))
-  (message "Done"))
 
+;;; replace.el ends here
