@@ -1,5 +1,5 @@
 ;; Process Emacs shell arguments
-;; Copyright (C) 1985-1993 Free Software Foundation, Inc.
+;; Copyright (C) 1985, 1986, 1990, 1992, 1993 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -17,49 +17,72 @@
 ;; along with GNU Emacs; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
-(defun command-line-process-help (arg)
-  (let ((standard-output (function external-debugging-output)))
+(defun command-line-do-help (arg)
+  "Print this message and exit."
+  (let ((standard-output 'external-debugging-output))
     (princ (emacs-version))
     (princ "\n\n")
     (if (fboundp 'x-create-screen)
-	(princ "Emacs accepts all standard X Toolkit command line options.\
+	(princ "Lucid Emacs accepts all standard X Toolkit command line options.\
   In addition,\nthe ")
       (princ "The "))
-    (princ "following options are processed in the order encountered:
-
-  -help			Print this message and exit.
-  -version		Print version info and exit.
-  -funcall <function>	Invoke the named lisp function with no arguments.
-  -f <function>		Same as -funcall.
-  -eval <form>		Evaluate the lisp form.  Quote it carefully.
-  -load <file>		Load the named file of lisp code into emacs.
-  -l <file>		Same as -load.
-  -insert <file>	Insert file into the current buffer.
-  -i <file>		Same as -insert.
-  -kill			Exit emacs.")
-    (if (featurep 'energize)
-    (princ "
-  -energize		Connect to the Energize server at $ENERGIZE_PORT."))
-    (princ "
-  +N <file>		Start displaying <file> at line N.
+    (princ "following options are processed in the order encountered:\n\n")
+    (let ((l command-switch-alist)
+	  (insert (function (lambda (&rest x)
+			      (princ "  ")
+			      (let ((len 2))
+				(while x
+				  (princ (car x))
+				  (setq len (+ len (length (car x))))
+				  (setq x (cdr x)))
+				(if (>= len 24)
+				    (progn (terpri) (setq len 0)))
+				(while (< len 24)
+				  (princ " ")
+				  (setq len (1+ len))))))))
+      (while l
+        (let ((name (car (car l)))
+              (fn (cdr (car l)))
+	      doc arg cons)
+	  (cond
+	   ((and (symbolp fn) (get fn 'undocumented)) nil)
+	   (t
+	    (setq doc (documentation fn))
+	    (if (member doc '(nil "")) (setq doc "(undocumented)"))
+	    (cond ((string-match "\n\\(<.*>\\)\n?\\'" doc)
+		   ;; Doc of the form "The frobber switch\n<arg1> <arg2>"
+		   (setq arg (substring doc (match-beginning 1) (match-end 1))
+			 doc (substring doc 0 (match-beginning 0))))
+		  ((string-match "\n+\\'" doc)
+		   (setq doc (substring doc 0 (match-beginning 0)))))
+	    (if (and (setq cons (rassq fn command-switch-alist))
+		     (not (eq cons (car l))))
+		(setq doc (format "Same as %s." (car cons))))
+	    (if arg
+		(funcall insert name " " arg)
+	      (funcall insert name))
+	    (princ doc)
+	    (terpri))))
+        (setq l (cdr l))))
+    (princ "\
+  +N <file>             Start displaying <file> at line N.
 
 These options are processed only if they appear before all other options:
 
-  -batch		Execute noninteractively (messages go to stderr.)
-			This option must be first in the list.
-  -nw			Inhibit the use of any window-system-specific
-			display code: use the current tty.
-  -no-init-file		Do not load an init file (~/.emacs).
-  -q			Same as -no-init-file.
-  -user <user>		Load user's init file instead of your own.
-  -u <user>		Same as -user.
+  -batch                Execute noninteractively (messages go to stderr.)
+                        This option must be first in the list.
+  -nw                   Inhibit the use of any window-system-specific
+                        display code: use the current tty.
+  -no-init-file         Do not load an init file (~/.emacs).
+  -q                    Same as -no-init-file.
+  -user <user>          Load user's init file instead of your own.
+  -u <user>             Same as -user.
 
 Anything else is considered a file name, and is placed into a buffer for
 editing.
 
 Emacs has an online tutorial and manuals.  Type ^Ht (Control-h t) after
-starting emacs to run the tutorial.  Type ^Hi to enter the manual browser.
-")
+starting emacs to run the tutorial.  Type ^Hi to enter the manual browser.\n")
     (kill-emacs 0)
     ))
 
@@ -71,8 +94,7 @@ starting emacs to run the tutorial.  Type ^Hi to enter the manual browser.
 
 
 ;; This should really be in files.el, but is used very early.
-(defvar directory-abbrev-alist
-  nil
+(defvar directory-abbrev-alist nil
   "*Alist of abbreviations for file directories.
 A list of elements of the form (FROM . TO), each meaning to replace
 FROM with TO when it appears in a directory name.
@@ -84,7 +106,7 @@ via absolute symbolic links.  Make TO the name of the link, and FROM
 the name it is linked to.")
 
 (defun abbreviate-file-name (filename &optional hack-homedir)
-  "Return a version of FILENAME shortened using directory-abbrev-alist.
+  "Return a version of FILENAME shortened using `directory-abbrev-alist'.
 See \\[describe-variable] directory-abbrev-alist RET for more information.
 If optional argument HACK-HOMEDIR is non-nil, then This also substitutes
 \"~\" for the user's home directory."
@@ -95,8 +117,11 @@ If optional argument HACK-HOMEDIR is non-nil, then This also substitutes
 		(concat (cdr (car tail)) (substring filename (match-end 0)))))
       (setq tail (cdr tail))))
   (if (and hack-homedir
-	   (string-match (concat "^" (regexp-quote (expand-file-name "~")))
-			 filename))
+	   (let ((hd (expand-file-name "~")))
+	     (if (string-match "/\\'" hd) ;lose trailing /
+		 (setq hd (substring hd 0 (match-beginning 0))))
+	     (and (< 0 (length hd)) ;homedir is root
+		  (string-match (concat "\\`" (regexp-quote hd)) filename))))
       (setq filename (concat "~" (substring filename (match-end 0)))))
   filename)
 
@@ -119,9 +144,19 @@ Elements look like (SWITCH-STRING . HANDLER-FUNCTION).
 HANDLER-FUNCTION receives switch name as sole arg;
 remaining command-line args are in the variable `command-line-args-left'.")
 
+(defvar before-init-hook nil
+  "Functions to call after handling urgent options but before loading init file.
+The window system interface uses this to open screens to display messages while
+Emacs loads the user's initialization file.")
+
+(defvar after-init-hook nil
+  "Functions to call after loading the init file (`~/.emacs').
+The call is not protected by a condition-case, so you can set `debug-on-error'
+in `.emacs', and put all the actual code on `after-init-hook'.")
+
 (defvar term-setup-hook nil
-  "Function to be called after loading terminal-specific lisp code.
-It is called with no arguments.  This variable exists for users to set,
+  "Functions to be called after loading terminal-specific lisp code.
+See `run-hooks'.  This variable exists for users to set,
 so as to override the definitions made by the terminal-specific file.
 Emacs never sets this variable itself.")
 
@@ -131,14 +166,17 @@ the proper function and keypad keys for use under X.  It is used in a
 fashion analogous to the environment value TERM.")
 
 (defvar window-setup-hook nil
-  "Function used to initialize window system display, after command line args.
-Users should not set this variable; use term-setup-hook instead.")
+  "Function called to initialize window system display.
+Emacs calls this after processing the command line arguments and loading
+the user's init file.
+
+Users should not set this variable; use `term-setup-hook' instead.")
 
 (defconst initial-major-mode 'lisp-interaction-mode
   "Major mode command symbol to use for the initial *scratch* buffer.")
 
 (defvar init-file-user nil
-  "When the `.emacs' file is read, this says which user's init file it is.
+  "Identity of user whose `.emacs' file is or was read.
 The value may be the null string or a string containing a user's name.
 If the value is a null string, it means that the init file was taken from
 the user that originally logged in.
@@ -146,30 +184,45 @@ the user that originally logged in.
 In all cases, `(concat \"~\" init-file-user \"/\")' evaluates to the
 directory name of the directory where the `.emacs' file was looked for.")
 
+(defvar init-file-debug nil)
+
 (defvar command-line-args-left) ; bound by `command-line'
 
-(defvar site-run-file "site-run"
+(defvar site-start-file (purecopy "site-start")
   "File containing site-wide run-time initializations.
-This file is loaded at run-time before ~/.emacs.  It contains inits
-that need to be in place for the entire site, but which, due to their
-higher incidence of change, don't make sense to load into emacs'
-dumped image.  Thus, the run-time load order is: 1. file described in
-this variable, if non-nil; 2. ~/.emacs; 3. default.el.")
+This file is loaded at run-time before ~/.emacs.
+It should contain initializations which need to be in place
+for the entire site, but which, due to their higher incidence
+of change, don't make sense to load into emacs' dumped image.
+
+Thus, the run-time load order is:
+  1. file named by this variable, if non-nil;
+  2. User's `.emacs';
+  3. default.el.")
 
 
 ;;; default switches
+;;; Note: these doc strings are semi-magical.
 
-(defun command-line-process-funcall (arg)
+(defun command-line-do-funcall (arg)
+  "Invoke the named lisp function with no arguments.
+<function>"
   (let ((fn (intern (car command-line-args-left))))
     (setq command-line-args-left (cdr command-line-args-left))
     (funcall fn)))
+(fset 'command-line-do-funcall-1 'command-line-do-funcall)
+(put 'command-line-do-funcall-1 'undocumented t)
 
-(defun command-line-process-eval (arg)
+(defun command-line-do-eval (arg)
+  "Evaluate the lisp form.  Quote it carefully.
+<form>"
   (let ((form (car command-line-args-left)))
     (setq command-line-args-left (cdr command-line-args-left))
     (eval (read form))))
 
-(defun command-line-process-load (arg)
+(defun command-line-do-load (arg)
+  "Load the named file of Lisp code into Emacs.
+<file>"
   (let ((file (car command-line-args-left)))
     ;; Take file from default dir if it exists there;
     ;; otherwise let `load' search for it.
@@ -178,82 +231,94 @@ this variable, if non-nil; 2. ~/.emacs; 3. default.el.")
     (load file nil t))
   (setq command-line-args-left (cdr command-line-args-left)))
 
-(defun command-line-process-insert (arg)
+(defun command-line-do-insert (arg)
+  "Insert file into the current buffer.
+<file>"
   (insert-file-contents (car command-line-args-left))
   (setq command-line-args-left (cdr command-line-args-left)))
 
-(defun command-line-process-kill (arg)
+(defun command-line-do-kill (arg)
+  "Exit Emacs."
   (kill-emacs t))
 
-(defun command-line-process-version (arg)
-  (princ (concat (emacs-version) "\n") (function external-debugging-output))
+(defun command-line-do-version (arg)
+  "Print version info and exit."
+  (princ (concat (emacs-version) "\n") 'external-debugging-output)
   (kill-emacs 0))
 
 (setq command-switch-alist
-      '(("-f"		. command-line-process-funcall)
-	("-e"		. command-line-process-funcall)
-	("-funcall"	. command-line-process-funcall)
-	("-eval"	. command-line-process-eval)
-	("-l"		. command-line-process-load)
-	("-load"	. command-line-process-load)
-	("-i"		. command-line-process-insert)
-	("-insert"	. command-line-process-insert)
-	("-kill"	. command-line-process-kill)
-	("-version"	. command-line-process-version)
-	("-help"	. command-line-process-help)
-	;; Options like +35 are handled specially.
-	;; Window-system, site, or package-specific code might add to this.
-	;; X11 handles its options by letting Xt remove args from this list.
-	))
+      (purecopy
+       '(("-help"	. command-line-do-help)
+	 ("-version"	. command-line-do-version)
+	 ("-funcall"	. command-line-do-funcall)
+         ("-f"		. command-line-do-funcall)
+	 ("-e"		. command-line-do-funcall-1)
+	 ("-eval"	. command-line-do-eval)
+	 ("-load"	. command-line-do-load)
+	 ("-l"		. command-line-do-load)
+	 ("-insert"	. command-line-do-insert)
+	 ("-i"		. command-line-do-insert)
+	 ("-kill"	. command-line-do-kill)
+	 ;; Options like +35 are handled specially.
+	 ;; Window-system, site, or package-specific code might add to this.
+	 ;; X11 handles its options by letting Xt remove args from this list.
+	 )))
 
-
 (defun premature-death-function (string &optional error)
-  (let ((stream (function external-debugging-output)))
-    (princ (if error 
-	       (format "\n%s: %s%s%s\n" string
-		       (get (car error) 'error-message)
-		       (if (cdr error) ": ")
-		       (mapconcat 'prin1-to-string (cdr error) ", "))
-	     string)
-	   stream)
+  (let ((stream 'external-debugging-output))
+    (if (null error)
+	(princ string stream)
+      (princ (concat "\n" string ": ") stream)
+      (display-error error stream)
+      (princ "\n" stream))
     (if (getenv "EMACSLOADPATH")
 	(princ (format "\n$EMACSLOADPATH is %s" (getenv "EMACSLOADPATH"))
 	       stream))
     (princ (format "\nload-path is %S" load-path) stream)
-    (princ (format "\nexec-directory is %S\n" exec-directory) stream))
+    (princ (format "\nexec-directory is %S\n" exec-directory) stream)
+    (princ (format "\ndata-directory is %S\n\n" data-directory) stream)
+    (backtrace stream t))
   (kill-emacs 33))
 
 (defun normal-top-level ()
   (if command-line-processed
       (message "Back to top level.")
     (setq command-line-processed t)
-    ;; In presence of symlinks, switch to cleaner form of default directory.
+    ;; Canonicalise HOME (PWD is canonicalised by init_buffer in buffer.c)
     (if (not (eq system-type 'vax-vms))
-	(mapcar (function
-		 (lambda (var)
-		   (let ((value (getenv var)))
-		     (if (and value
-			      (< (length value) (length default-directory))
-			      (equal (file-attributes default-directory)
-				     (file-attributes value)))
-			 (setq default-directory
-			       (file-name-as-directory value))))))
-		'("PWD" "HOME")))
-    (let ((tail directory-abbrev-alist))
-      (while tail
-	(if (string-match (car (car tail)) default-directory)
-	    (setq default-directory
-		  (concat (cdr (car tail))
-			  (substring default-directory (match-end 0)))))
-	(setq tail (cdr tail))))
-    (command-line)))
-
+        (let ((value (getenv "HOME")))
+          (if (and value
+                   (< (length value) (length default-directory))
+                   (equal (file-attributes default-directory)
+                          (file-attributes value)))
+              (setq default-directory (file-name-as-directory value)))))
+    (setq default-directory (abbreviate-file-name default-directory))
+    (unwind-protect
+         (command-line)
+      (run-hooks 'emacs-startup-hook)
+      (run-hooks 'term-setup-hook)
+      (setq term-setup-hook nil)
+      (run-hooks 'window-setup-hook)
+      (setq window-setup-hook nil))))
 
 (defun command-line-init ()
+  ;; See if we should import version-control from the environment variable.
+  (let ((vc (getenv "VERSION_CONTROL")))
+    (and vc (cond
+	     ((or (string= vc "t")
+		  (string= vc "numbered"))
+	      (setq version-control t))
+	     ((or (string= vc "nil")
+		  (string= vc "existing"))
+	      (setq version-control nil))
+	     ((or (string= vc "never")
+		  (string= vc "simple"))
+	      (setq version-control 'never)))))
   (let ((done nil))
     ;; If user has not done su, use current $HOME to find .emacs.
     (and init-file-user (string= init-file-user (user-real-login-name))
 	 (setq init-file-user ""))
+
     (while (and (not done) command-line-args-left)
       (let ((argi (car command-line-args-left)))
 	(cond ((or (string-equal argi "-q")
@@ -261,26 +326,17 @@ this variable, if non-nil; 2. ~/.emacs; 3. default.el.")
 	       (setq init-file-user nil
 		     command-line-args-left (cdr command-line-args-left)))
 	      ((string-equal argi "-no-site-file")
-	       (setq site-run-file nil
+	       (setq site-start-file nil
 		     command-line-args-left (cdr command-line-args-left)))
 	      ((or (string-equal argi "-u")
 		   (string-equal argi "-user"))
 	       (setq command-line-args-left (cdr command-line-args-left)
 		     init-file-user (car command-line-args-left)
 		     command-line-args-left (cdr command-line-args-left)))
- 	      (t (setq done t)))))
-    (let ((vc (getenv "VERSION_CONTROL")))
-      (and vc (cond
-	       ((or (string= vc "t")
-		    (string= vc "numbered"))
-		(setq version-control t))
-	       ((or (string= vc "nil")
-		    (string= vc "existing"))
-		(setq version-control nil))
-	       ((or (string= vc "never")
-		    (string= vc "simple"))
-		(setq version-control 'never)))))
-    ))
+              ((string-equal argi "-debug-init")
+               (setq init-file-debug t
+                     command-line-args-left (cdr command-line-args-left)))
+ 	      (t (setq done t)))))))
 
 
 (defun command-line ()
@@ -288,8 +344,9 @@ this variable, if non-nil; 2. ~/.emacs; 3. default.el.")
     (condition-case error
       (progn
 	(set-default-load-path)
-	(setq init-file-user (if noninteractive nil (user-login-name)))
-	;;
+
+	(setq init-file-user (if (noninteractive) nil (user-login-name)))
+
 	;; When running emacs under a window system, the window-system-specific
 	;; files are loaded and hooks are run before the user's init file is
 	;; loaded; this is so that the user can see messages that the init file
@@ -310,19 +367,35 @@ this variable, if non-nil; 2. ~/.emacs; 3. default.el.")
 	;; the case, then there would be no need for the term-init-hook (the
 	;; init file could do it directly.)
 	;;
-	(if window-system (initialize-terminal-1))
+	(if window-system
+            (condition-case error
+                (progn
+                  (load (concat term-file-prefix
+                                (symbol-name window-system)
+                                "-win")
+                        ;; Every window system should have a startup file;
+                        ;; barf if we can't find it.
+                        nil t)
+                  ;; initialize the window system, 
+                  ;;  create the first screen, etc.
+                  (run-hooks 'window-setup-hook)
+                  (setq window-setup-hook nil))
+              (error
+               (premature-death-function "Error in window-setup" error))))
 
-	(if (and (eq window-system 'x)
-		 (null (x-window-id (selected-screen))))
-	    (premature-death-function
-	"Initialization error: Loading term/x-win.el didn't create an X screen!
+        (and (eq window-system 'x)
+             (null (x-window-id (selected-screen)))
+             (premature-death-function 
+  "Initialization error: Loading term/x-win.el didn't create an X screen!
 This probably means that this emacs is picking up an old (v18) lisp directory.
 "))
+
 	;; process magic command-line switches like -q and -u.
 	(command-line-init)
 
 	;; initialize redisplay to make Fmessage() work.
-	(initialize-first-screen)
+        ;; >>> shouldn't window-setup-hook do this?
+	(or (noninteractive) (initialize-first-screen))
 	)
       ;;
       ;; If we get an error above, it's almost always because emacs couldn't
@@ -335,6 +408,7 @@ This probably means that this emacs is picking up an old (v18) lisp directory.
       ;;
       (error
        (premature-death-function "Initialization error" error)))
+
     ;;
     ;; We have normality, I repeat, we have normality.  Anything you still
     ;; can't cope with is therefore your own problem.  (And we don't need
@@ -349,85 +423,78 @@ This probably means that this emacs is picking up an old (v18) lisp directory.
 	  (if (eq major-mode 'fundamental-mode)
 	      (funcall initial-major-mode))))
 
-    ;; Initialize terminal (not window system.)  See comment above.
-    (or window-system (initialize-terminal-1))
-    
-    ;; run user's terminal init hooks.
-    (initialize-terminal-2)
+    ;; Initialize terminal (not window system.)  See comment about the
+    ;; crockishness of this above.
+    (and (not window-system)
+         ;; Load library for our terminal type or window system.
+         ;; User init file can set term-file-prefix to nil to prevent this.
+         term-file-prefix
+         (not (noninteractive))
+         (let ((term (getenv "TERM"))
+               hyphend)
+           (while (and term
+                       (not (load (concat term-file-prefix term) t t)))
+             ;; Strip off last hyphen and what follows, then try again
+             (if (setq hyphend (string-match "[-_][^-_]+\\'" term))
+                 (setq term (substring term 0 hyphend))
+                 (setq term nil)))))
+
+    ;; run the user's terminal init hooks.
+    (condition-case error
+        (run-hooks 'term-setup-hook)
+      (error
+       (message "Error in term-setup-hook: ")
+       (display-error error nil)
+       (sit-for 1)))
+    ;; Don't let the hook be run twice.
+    (setq term-setup-hook nil)
     
     ;; now process the rest of the command line, including user options.
     (command-line-1)
     
-    (if noninteractive (kill-emacs t))))
+    (if (noninteractive) (kill-emacs t))))
 
 
 ;;; Load user's init file and default ones.
 (defun load-init-file ()
-  (condition-case error
-      (progn
- 	;; load site-wide run-time init file first
-	(let ((inhibit-startup-message nil))
-	  (and (stringp site-run-file)
-	       (load site-run-file t t)))
-	(if init-file-user
-	    (progn (load (if (eq system-type 'vax-vms)
-			     "sys$login:.emacs"
-			   (concat "~" init-file-user "/.emacs"))
-			 t t t)
-		   (or inhibit-default-init
-		       (let ((inhibit-startup-message nil))
-			 ;; Users are supposed to be told their rights.
-			 ;; (Plus how to get help and how to undo.)
-			 ;; Don't you dare turn this off for anyone
-			 ;; except yourself.
-			 (load "default" t t))))))
-    (error (message "Error in init file: %s%s%s"
-		    (get (car error) 'error-message)
-		    (if (cdr error) ": ")
-		    (mapconcat 'prin1-to-string (cdr error) ", ")))))
+  (run-hooks 'before-init-hook)
 
-(defun initialize-terminal-1 ()
-  ;; Load library for our terminal type or window system.
-  ;; User init file can set term-file-prefix to nil to prevent this.
-  (and term-file-prefix (not noninteractive)
-       (if window-system
-	   (load (concat term-file-prefix
-			 (symbol-name window-system)
-			 "-win")
-		 ;; Every window system should have a startup file;
-		 ;; barf if we can't find it.
-		 nil t)
-	 ;; else
-	 (let ((term (getenv "TERM"))
-	       hyphend)
-	   (while (and term
-		       (not (load (concat term-file-prefix term) t t)))
-	     ;; Strip off last hyphen and what follows, then try again
-	     (if (setq hyphend (string-match "[-_][^-_]+$" term))
-		 (setq term (substring term 0 hyphend))
-	       (setq term nil))))))
+  ;; Run the site-start library if it exists.  The point of this file is
+  ;; that it is run before .emacs.  There is no point in doing this after
+  ;; .emacs; that is useless.
+  (let ((inhibit-startup-message nil))
+    (and (stringp site-start-file)
+         (load site-start-file t t)))
 
-  ;; initialize the window system, create the first screen, etc.
-  (condition-case error
-      (and window-setup-hook
-	   (run-hooks 'window-setup-hook))
-    (error
-     (premature-death-function "Error in window-setup-hook" error))))
-
-(defun initialize-terminal-2 ()
-  ;; run the user's terminal init hooks.
-  (condition-case error
-      (and term-setup-hook
-	   (run-hooks 'term-setup-hook))
-    (error (message "Error in term-setup-hook: %s%s%s"
-		    (get (car error) 'error-message)
-		    (if (cdr error) ": ")
-		    (mapconcat 'prin1-to-string (cdr error) ", ")))))
+  (let ((load-init-file
+          (function (lambda ()
+            (if init-file-user
+                (progn (load (if (eq system-type 'vax-vms)
+                                 "sys$login:.emacs"
+			       (concat "~" init-file-user "/.emacs"))
+                             t t t)
+                       (or inhibit-default-init
+                           (let ((inhibit-startup-message nil))
+                             ;; Users are supposed to be told their rights.
+                             ;; (Plus how to get help and how to undo.)
+                             ;; Don't you dare turn this off for anyone
+                             ;; except yourself.
+                             (load "default" t t)))))))))
+    ;; Load that user's init file, or the default one, or none.
+    (if init-file-debug
+        (let ((debug-on-error t))
+          (funcall load-init-file))
+        (condition-case error
+            (funcall load-init-file)
+          (error
+           (message "Error in init file: ")
+           (display-error error nil)))))
 
+    (run-hooks 'after-init-hook))
 
 (defun command-line-1 ()
   (if (null command-line-args-left)
-      (cond ((and (not inhibit-startup-message) (not noninteractive)
+      (cond ((and (not inhibit-startup-message) (not (noninteractive))
 		  ;; Don't clobber a non-scratch buffer if init file
 		  ;; has selected it.
 		  (string= (buffer-name) "*scratch*")
@@ -446,6 +513,7 @@ more information about Energize, or about Lucid Emacs support.")
        "\n\nType \\[help-command] for help; \\[advertised-undo] to undo changes.  (`C-' means use CTRL key.)
 To kill the Emacs job, type \\[save-buffers-kill-emacs].
 Type \\[help-with-tutorial] for a tutorial on using Emacs.
+Type \\[info] to enter Info, which you can use to read GNU documentation.
 
 GNU Emacs comes with ABSOLUTELY NO WARRANTY; type \\[describe-no-warranty] for full details.
 You may give out copies of Emacs; type \\[describe-copying] to see the conditions.
@@ -462,30 +530,45 @@ Type \\[describe-distribution] for information on getting the latest version."))
     (let ((dir default-directory)
 	  (file-count 0)
 	  first-file-buffer
-	  (line 0))
+	  (line nil))
       (while command-line-args-left
 	(let ((argi (car command-line-args-left))
 	      tem)
 	  (setq command-line-args-left (cdr command-line-args-left))
-	  (cond ((setq tem (assoc argi command-switch-alist))
-		 (funcall (cdr tem) argi))
-		((string-match "^\\+[0-9]+\\'" argi)
-		 (setq line (string-to-int argi)))
-		(t
-		 (setq file-count (1+ file-count))
-		 (cond ((= file-count 1)
-			(setq first-file-buffer
-			      (progn
-				(find-file (expand-file-name argi dir))
-				(current-buffer))))
-		       (t
-			(find-file-other-window (expand-file-name argi dir))))
-		 (or (zerop line)
-		     (goto-line line))
-		 (setq line 0)))))
-      ;; If 3 or more files visited, and not all visible, show user what
-      ;; they all are.
-      (if (> file-count 2)
+	  (or (cond (line 
+                     nil)
+                    ((setq tem (or (assoc argi command-switch-alist)
+                                   (and (string-match "\\`--" argi)
+                                        (assoc (substring argi 1)
+                                               command-switch-alist))))
+                     (funcall (cdr tem) argi)
+                     t)
+                    ((string-match "\\`\\+[0-9]+\\'" argi)
+                     (setq line (string-to-int argi))
+                     t)
+                    ((or (equal argi "-") (equal argi "--"))
+                     ;; "- file" means don't treat "file" as a switch
+                     ;;  ("+0 file" has the same effect; "-" added
+                     ;;   for unixoidiality.)
+		     ;; This is worthless; the `unixoid' way is "./file". -jwz
+                     (setq line 0))
+                    (t
+                     nil))
+              (progn
+                (setq file-count (1+ file-count))
+                (setq argi (expand-file-name argi dir))
+                (if (= file-count 1)
+                    (setq first-file-buffer (progn (find-file argi)
+                                                   (current-buffer)))
+		  (if noninteractive
+		      (find-file argi)
+                    (find-file-other-window argi)))
+                (goto-line (or line 0))
+		(setq line nil)))))
+      ;; If 3 or more files visited, and not all visible,
+      ;; show user what they all are.
+      (if (and (not noninteractive)
+	       (> file-count 2))
 	  (or (get-buffer-window first-file-buffer)
 	      (progn (other-window 1)
 		     (buffer-menu nil)))))))
@@ -516,6 +599,10 @@ Type \\[describe-distribution] for information on getting the latest version."))
 	  (find-emacs-root-internal (expand-file-name path dir)))
      ;;
      ;; Otherwise, this directory just doesn't cut it.
+     ;; Some bozos think they can use the 18.59 lisp directory with 19.*.
+     ;; This is because they're not using their brains.  But it might be
+     ;; nice to notice that that is happening and point them in the
+     ;; general direction of a clue.
      ;;
      nil)))
 
@@ -556,11 +643,13 @@ Type \\[describe-distribution] for information on getting the latest version."))
 	    (setq exec-path (append exec-path (list etc)))))
     (if (and (null exec-directory) etc)
 	(setq exec-directory (file-name-as-directory etc)))
-    ;; Default the info dir to being a sibling of the exec-directory.
-    (if (and (boundp 'Info-directory-list) (null Info-directory-list))
+    (if (and (null data-directory) etc)
+	(setq data-directory (file-name-as-directory etc)))
+    ;; Default the info dir to being a sibling of the data-directory.
+    (if (or (not (boundp 'Info-directory-list)) (null Info-directory-list))
 	(setq Info-directory-list
-	      (list (expand-file-name "../info/" exec-directory))))
-    ;; Default the lock dir to being a sibling of the exec-directory.
+	      (list (expand-file-name "../info/" data-directory))))
+    ;; Default the lock dir to being a sibling of the data-directory.
     ;; If superlock isn't set, derive it from the lock dir.
     (if (boundp 'lock-directory)
 	(progn
@@ -587,6 +676,7 @@ Type \\[describe-distribution] for information on getting the latest version."))
 	    (setq warnings (cons "lock-directory" warnings)))
 	(if (null exec-directory)
 	    (setq warnings (cons "exec-directory" warnings)))
+        ;>>>> data-directory
 	(if (null load-path)
 	    (setq warnings (cons "load-path" warnings)))
 	(cond ((cdr (cdr warnings))
@@ -600,14 +690,15 @@ Type \\[describe-distribution] for information on getting the latest version."))
 		"emacs executable, " execution-path " is in a strange place?")
 	(setq guess (or exec-directory
 			(car (reverse load-path))
-			(and (string-match "/[^/]+$" execution-path)
+			(and (string-match "/[^/]+\\'" execution-path)
 			     (substring execution-path 0
 					(match-beginning 0)))))
-	(if (and guess (string-match "/\\(src\\|etc\\|lisp\\)/?$" guess))
+	(if (and guess (string-match "/\\(src\\|etc\\|lisp\\)/?\\'" guess))
 	    (setq guess (substring guess 0 (match-beginning 0))))
-	(if (and guess (string-match "/$" guess))
+	(if (and guess (string-match "/\\'" guess))
 	    (setq guess (substring guess 0 (match-beginning 0))))
 
+        ;>>>> data-directory
 	(if (or (null exec-directory) (null load-path))
 	    (insert
 	     "\n\nWithout both exec-directory and load-path, emacs will "
@@ -633,14 +724,17 @@ Type \\[describe-distribution] for information on getting the latest version."))
 		   "symbolic link for use as the lock directory.  "
 		   "(This directory must be globally writable.)"))))
 
-	(fill-region (point-min) (point-max))
+        (if (fboundp 'fill-region)
+            ;; Might not be bound in the cold load environment...
+	    (fill-region (point-min) (point-max)))
 	(goto-char (point-min))
-	(princ "\nWARNING:\n" (function external-debugging-output))
-	(princ (buffer-string) (function external-debugging-output))
+	(princ "\nWARNING:\n" 'external-debugging-output)
+	(princ (buffer-string) 'external-debugging-output)
 	(erase-buffer)
 	t)))))
 
 
+;;>>> Pathetic variable name.  And undocumented to boot!
 (defvar cdlist nil)
 
 (defun initialize-cdlist ()
@@ -661,6 +755,7 @@ Type \\[describe-distribution] for information on getting the latest version."))
 	    (setq l (cons (directory-file-name
 			   (buffer-substring here (1- (point)))) l)
 		  here (point)))
+          (kill-buffer (current-buffer))
 	  (nreverse l))
       nil)))
 
