@@ -1,5 +1,5 @@
 /* Simple built-in editing commands.
-   Copyright (C) 1985, 1992 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1992, 1993 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -24,50 +24,61 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "commands.h"
 #include "buffer.h"
 #include "syntax.h"
+#include "insdel.h"
 
-Lisp_Object Qkill_forward_chars, Qkill_backward_chars, Vblink_paren_function;
+Lisp_Object Qkill_forward_chars;
+Lisp_Object Qself_insert_command;
 
-extern int zmacs_region_stays;
+Lisp_Object Vblink_paren_function;
+
+/* A possible value for a buffer's overwrite-mode variable.  */
+Lisp_Object Qoverwrite_mode_binary;
 
 
-DEFUN ("forward-char", Fforward_char, Sforward_char, 0, 1, "p",
+DEFUN ("forward-char", Fforward_char, Sforward_char, 0, 1, "_p",
   "Move point right ARG characters (left if ARG negative).\n\
 On reaching end of buffer, stop and signal error.")
   (n)
      Lisp_Object n;
 {
-  register int to;
-
   if (NILP (n))
-    XFASTINT (n) = 1;
+    n = make_number (1);
   else
     CHECK_FIXNUM (n, 0);
 
-  to = (point + XINT (n));
-  if (to < BEGV)
-    {
-      SET_PT (BEGV);
-      Fsignal (Qbeginning_of_buffer, Qnil);
-    }
-  if (to > ZV)
-    {
-      SET_PT (ZV);
-      Fsignal (Qend_of_buffer, Qnil);
-    }
+  /* This used to just set point to point + XINT (n), and then check
+     to see if it was within boundaries.  But now that SET_PT can
+     potentially do a lot of stuff (calling entering and exiting
+     hooks, etcetera), that's not a good approach.  So we validate the
+     proposed position, then set point.  */
+  {
+    int new_point = point + XINT (n);
+
+    if (new_point < BEGV)
+      {
+	SET_PT (BEGV);
+	Fsignal (Qbeginning_of_buffer, Qnil);
+      }
+    if (new_point > ZV)
+      {
+	SET_PT (ZV);
+	Fsignal (Qend_of_buffer, Qnil);
+      }
+
+    SET_PT (new_point);
+  }
 
-  SET_PT (to);
-  zmacs_region_stays = 1;
   return Qnil;
 }
 
-DEFUN ("backward-char", Fbackward_char, Sbackward_char, 0, 1, "p",
+DEFUN ("backward-char", Fbackward_char, Sbackward_char, 0, 1, "_p",
   "Move point left ARG characters (right if ARG negative).\n\
 On attempt to pass beginning or end of buffer, stop and signal error.")
   (n)
      Lisp_Object n;
 {
   if (NILP (n))
-    XFASTINT (n) = 1;
+    n = make_number (1);
   else
     CHECK_FIXNUM (n, 0);
 
@@ -75,7 +86,7 @@ On attempt to pass beginning or end of buffer, stop and signal error.")
   return Fforward_char (n);
 }
 
-DEFUN ("forward-line", Fforward_line, Sforward_line, 0, 1, "p",
+DEFUN ("forward-line", Fforward_line, Sforward_line, 0, 1, "_p",
   "Move ARG lines forward (backward if ARG is negative).\n\
 Precisely, if point is on line I, move to the start of line I + ARG.\n\
 If there isn't room, go as far as possible (no error).\n\
@@ -103,15 +114,15 @@ With positive ARG, a non-empty line at the end counts as one line\n\
   if (shortage > 0
       && (negp
 	  || (ZV > BEGV
-	      && CHAR_AT (pos - 1) != '\n')))
+	      && pos != pos2
+	      && FETCH_CHAR (pos - 1) != '\n')))
     shortage--;
   SET_PT (pos);
-  zmacs_region_stays = 1;
   return make_number (negp ? - shortage : shortage);
 }
 
 DEFUN ("beginning-of-line", Fbeginning_of_line, Sbeginning_of_line,
-  0, 1, "p",
+  0, 1, "_p",
   "Move point to beginning of current line.\n\
 With argument ARG not nil or 1, move forward ARG - 1 lines first.\n\
 If scan reaches end of buffer, stop there without error.")
@@ -119,7 +130,7 @@ If scan reaches end of buffer, stop there without error.")
      Lisp_Object n;
 {
   if (NILP (n))
-    XFASTINT (n) = 1;
+    n = make_number (1);
   else
     CHECK_FIXNUM (n, 0);
 
@@ -128,7 +139,7 @@ If scan reaches end of buffer, stop there without error.")
 }
 
 DEFUN ("end-of-line", Fend_of_line, Send_of_line,
-  0, 1, "p",
+  0, 1, "_p",
   "Move point to end of current line.\n\
 With argument ARG not nil or 1, move forward ARG - 1 lines first.\n\
 If scan reaches end of buffer, stop there without error.")
@@ -139,7 +150,7 @@ If scan reaches end of buffer, stop there without error.")
   register int stop;
 
   if (NILP (n))
-    XFASTINT (n) = 1;
+    n = make_number (1);
   else
     CHECK_FIXNUM (n, 0);
 
@@ -148,9 +159,8 @@ If scan reaches end of buffer, stop there without error.")
 
   pos = point;
   stop = ZV;
-  while (pos < stop && CHAR_AT (pos) != '\n') pos++;
+  while (pos < stop && FETCH_CHAR (pos) != '\n') pos++;
   SET_PT (pos);
-  zmacs_region_stays = 1;
   return Qnil;
 }
 
@@ -169,14 +179,14 @@ ARG was explicitly specified.")
       if (XINT (n) < 0)
 	{
 	  if (point + XINT (n) < BEGV)
-	    Fsignal (Qbeginning_of_buffer, Qnil);
+	    signal_error (Qbeginning_of_buffer, Qnil);
 	  else
 	    del_range (point + XINT (n), point);
 	}
       else
 	{
 	  if (point + XINT (n) > ZV)
-	    Fsignal (Qend_of_buffer, Qnil);
+	    signal_error (Qend_of_buffer, Qnil);
 	  else
 	    del_range (point, point + XINT (n));
 	}
@@ -201,7 +211,7 @@ ARG was explicitly specified.")
   return Fdelete_char (make_number (-XINT (n)), killflag);
 }
 
-extern Lisp_Object Vlast_command_char, Vlast_command_event;
+static void internal_self_insert (int ch, int noautofill);
 
 DEFUN ("self-insert-command", Fself_insert_command, Sself_insert_command, 1, 1, "p",
   "Insert the character you type.\n\
@@ -209,17 +219,27 @@ Whichever character you type to run this command is inserted.")
   (arg)
      Lisp_Object arg;
 {
+  int n, ch;
+  Lisp_Object c;
   CHECK_FIXNUM (arg, 0);
 
-  if (NILP (Vlast_command_char))
-    Fsignal (Qerror,
-	     Fcons (build_string
-		    ("last typed character has no ASCII equivalent"),
-		    Fcons (Fcopy_event (Vlast_command_event, Qnil), Qnil)));
-  while (XINT (arg) > 0)
+  if (FIXNUMP (Vlast_command_char))
+    c = Vlast_command_char;
+  else
+    c = Fevent_to_character (Vlast_command_event, Qnil, Qnil, Qt);
+
+  if (NILP (c))
+    signal_error (Qerror,
+                  list2 (build_string (
+			 "last typed character has no ASCII equivalent"),
+                         Fcopy_event (Vlast_command_event, Qnil)));
+
+  n = XINT (arg);
+  ch = XINT (c);
+  while (n > 0)
     {
-      XFASTINT (arg)--;		/* Ok since old and new vals both nonneg */
-      internal_self_insert (Vlast_command_char, XFASTINT (arg) != 0);
+      n--;
+      internal_self_insert (ch, (n != 0));
     }
   return Qnil;
 }
@@ -231,33 +251,34 @@ In Auto Fill mode, if no numeric arg, break the preceding line if it's long.")
      Lisp_Object arg1;
 {
   int flag;
-  Lisp_Object arg;
+  int arg;
   char c1 = '\n';
 
-  arg = Fprefix_numeric_value (arg1);
+  arg = XINT (Fprefix_numeric_value (arg1));
 
   if (!NILP (current_buffer->read_only))
-    Fsignal (Qbuffer_read_only, Qnil);
+    Fbarf_if_buffer_read_only ();
 
-  /* Inserting a newline at the end of a line
-     produces better redisplay in try_window_id
-     than inserting at the ebginning fo a line,
-     And the textual result is the same.
-     So if at beginning, pretend to be at the end.
-     Must avoid internal_self_insert in that case since point is wrong.
-     Luckily internal_self_insert's special features all do nothing in that case.  */
+  /* Inserting a newline at the end of a line produces better
+     redisplay in try_window_id than inserting at the beginning of a
+     line, and the textual result is the same.  So, if we're at
+     beginning of line, pretend to be at the end of the previous line.  
 
-  flag = point > BEGV && CHAR_AT (point - 1) == '\n';
+     We can't use internal_self_insert in that case since it won't do
+     the insertion correctly.  Luckily, internal_self_insert's special
+     features all do nothing in that case.  */
+
+  flag = point > BEGV && FETCH_CHAR (point - 1) == '\n';
   if (flag)
     SET_PT (point - 1);
 
-  while (XINT (arg) > 0)
+  while (arg > 0)
     {
       if (flag)
 	insert_raw_string (&c1, 1);
       else
 	internal_self_insert ('\n', !NILP (arg1));
-      XFASTINT (arg)--;		/* Ok since old and new vals both nonneg */
+      arg--;
     }
 
   if (flag)
@@ -266,15 +287,20 @@ In Auto Fill mode, if no numeric arg, break the preceding line if it's long.")
   return Qnil;
 }
 
-internal_self_insert (c1, noautofill)
-     char c1;
-     int noautofill;
+/* Insert character C1.  If NOAUTOFILL is nonzero, don't do autofill
+   even if it is enabled.
+
+   If this insertion is suitable for direct output (completely simple),
+   return 0.  A value of 1 indicates this *might* not have been simple.  */
+
+static void
+internal_self_insert (int c1, int noautofill)
 {
-  extern Lisp_Object Fexpand_abbrev ();
-  int hairy = 0;
-  Lisp_Object tem;
+  /* int hairy = 0; -- unused */
   register enum syntaxcode synt;
   register int c = c1;
+  Lisp_Object overwrite = current_buffer->overwrite_mode;
+  Lisp_Object syntax_table = current_buffer->syntax_table;
 
 #if 0
   /* No, this is very bad, it makes undo *always* undo a character at a time
@@ -284,58 +310,64 @@ internal_self_insert (c1, noautofill)
     hairy = 1;
 #endif
 
-  if (!NILP (current_buffer->overwrite_mode)
+  if (!NILP (overwrite)
       && point < ZV
-      && c != '\n' && CHAR_AT (point) != '\n'
-      && (CHAR_AT (point) != '\t'
+      && (EQ (overwrite, Qoverwrite_mode_binary)
+	  || (c != '\n' && FETCH_CHAR (point) != '\n'))
+      && (EQ (overwrite, Qoverwrite_mode_binary)
+          || FETCH_CHAR (point) != '\t'
 	  || XINT (current_buffer->tab_width) <= 0
-	  || !((current_column () + 1) % XFASTINT (current_buffer->tab_width))))
+	  || XINT (current_buffer->tab_width) > 20
+	  || !((current_column () + 1) % XINT (current_buffer->tab_width))))
     {
       del_range (point, point + 1);
-      hairy = 1;
+      /* hairy = 1; */
     }
   if (!NILP (current_buffer->abbrev_mode)
-      && SYNTAX (c) != Sword
+      && SYNTAX (syntax_table, c) != Sword
       && NILP (current_buffer->read_only)
-      && point > BEGV && SYNTAX (CHAR_AT (point - 1)) == Sword)
+      && point > BEGV
+      && SYNTAX (syntax_table, FETCH_CHAR (point - 1)) == Sword)
     {
-      tem = Fexpand_abbrev ();
-      if (!NILP (tem))
-	hairy = 1;
+      /* int modiff = MODIFF; */
+      Fexpand_abbrev ();
+      /* We can't trust the value of Fexpand_abbrev,
+	 but if Fexpand_abbrev changed the buffer,
+	 assume it expanded something.  */
+      /* if (MODIFF != modiff)
+	hairy = 1; */
     }
   if ((c == ' ' || c == '\n')
       && !noautofill
       && !NILP (current_buffer->auto_fill_function)
-      && current_column () > XFASTINT (current_buffer->fill_column))
+      && current_column () > XINT (current_buffer->fill_column))
     {
       if (c1 != '\n')
-	insert_raw_string (&c1, 1);
+	insert_char (c1);
       call0 (current_buffer->auto_fill_function);
       if (c1 == '\n')
-	insert_raw_string (&c1, 1);
-      hairy = 1;
+	insert_char (c1);
+      /* hairy = 1; */
     }
   else
-    insert_raw_string (&c1, 1);
-  synt = SYNTAX (c);
+    insert_char (c1);
+  synt = SYNTAX (syntax_table, c);
   if ((synt == Sclose || synt == Smath)
       && !NILP (Vblink_paren_function) && INTERACTIVE)
     {
       call0 (Vblink_paren_function);
-      hairy = 1;
+      /* hairy = 1; */
     }
-  return hairy;
 }
 
 /* module initialization */
 
+void
 syms_of_cmds ()
 {
-  Qkill_backward_chars = intern ("kill-backward-chars");
-  staticpro (&Qkill_backward_chars);
-
-  Qkill_forward_chars = intern ("kill-forward-chars");
-  staticpro (&Qkill_forward_chars);
+  defsymbol (&Qkill_forward_chars, "kill-forward-chars");
+  defsymbol (&Qself_insert_command, "self-insert-command");
+  defsymbol (&Qoverwrite_mode_binary, "overwrite-mode-binary");
 
   DEFVAR_LISP ("blink-paren-function", &Vblink_paren_function,
     "Function called, if non-nil, whenever a close parenthesis is inserted.\n\
@@ -354,20 +386,3 @@ More precisely, a char with closeparen syntax is self-inserted.");
   defsubr (&Sself_insert_command);
   defsubr (&Snewline);
 }
-
-keys_of_cmds ()
-{
-  int n;
-
-  initial_define_key (global_map, Ctl('M'), "newline");
-  initial_define_key (global_map, Ctl('I'), "self-insert-command");
-  for (n = 040; n < 0177; n++)
-    initial_define_key (global_map, n, "self-insert-command");
-
-  initial_define_key (global_map, Ctl ('A'), "beginning-of-line");
-  initial_define_key (global_map, Ctl ('B'), "backward-char");
-  initial_define_key (global_map, Ctl ('D'), "delete-char");
-  initial_define_key (global_map, Ctl ('E'), "end-of-line");
-  initial_define_key (global_map, Ctl ('F'), "forward-char");
-  initial_define_key (global_map, 0177, "delete-backward-char");
-}
