@@ -1,5 +1,6 @@
 /* Indentation functions.
-   Copyright (C) 1992, 1993 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1987, 1988, 1992, 1993 
+   Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -22,15 +23,18 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "lisp.h"
 #include "buffer.h"
 #include "extents.h"
+#include "insdel.h"
 #include "indent.h"
 #include "screen.h"
 #include "window.h"
-#include "insdel.h"
 #include "termchar.h"
 #include "termopts.h"
 #include "disptab.h"
+#include "dispmisc.h"
+#include "faces.h"
 #ifdef HAVE_X_WINDOWS
 #include "xterm.h"
+#include "xobjs.h"
 #endif
 
 /* Indentation can insert tabs if this is non-zero;
@@ -48,26 +52,23 @@ int indent_tabs_mode;
 
    Some things set last_known_column_point to -1
    to mark the memoized value as invalid */
-int last_known_column;
+static int last_known_column;
 
 /* Last buffer searched by current_column */
 static struct buffer *last_known_column_buffer;
 
 /* Value of point when current_column was called */
-int last_known_column_point;
+static int last_known_column_point;
 
 /* Value of MODIFF when current_column was called */
 static int last_known_column_modified;
 
-extern int minibuf_prompt_width;
-
-extern int minibuf_prompt_pix_width;
-
 /* Get the display table to use for the current buffer.  */
 
 struct Lisp_Vector *
 buffer_display_table (struct buffer* buffer)
 {
+#if 0 /* >>> Unused */
   Lisp_Object thisbuf;
 
   thisbuf = buffer->display_table;
@@ -78,6 +79,7 @@ buffer_display_table (struct buffer* buffer)
   if (VECTORP (Vstandard_display_table)
       && XVECTOR (Vstandard_display_table)->size == DISP_TABLE_SIZE)
     return XVECTOR (Vstandard_display_table);
+#endif
 
   return 0;
 }
@@ -98,6 +100,14 @@ list_length (list)
 }
 #endif
 
+/* Cancel any recorded value of the horizontal position.  */
+ 
+void
+invalidate_current_column ()
+{
+  last_known_column_point = -1;
+}
+
 int
 current_column ()
 {
@@ -171,9 +181,7 @@ Whether the line is visible (if `selective-display' is t) has no effect;\n\
 however, ^M is treated as end of line when `selective-display' is t.")
   ()
 {
-  Lisp_Object temp;
-  XFASTINT (temp) = current_column ();
-  return temp;
+  return (make_number (current_column ()));
 }
 
 DEFUN ("indent-to", Findent_to, Sindent_to, 1, 2, "NIndent to column: ",
@@ -191,8 +199,9 @@ even if that goes past COLUMN; by default, MIN is zero.")
 
   CHECK_FIXNUM (col, 0);
   if (NILP (minimum))
-    XFASTINT (minimum) = 0;
-  CHECK_FIXNUM (minimum, 1);
+    minimum = Qzero;
+  else
+    CHECK_FIXNUM (minimum, 1);
 
   fromcol = current_column ();
   mincol = fromcol + XINT (minimum);
@@ -217,18 +226,16 @@ even if that goes past COLUMN; by default, MIN is zero.")
 
   if (indent_tabs_mode)
     {
-      Lisp_Object n;
-      XFASTINT (n) = mincol / tab_width - fromcol / tab_width;
-      if (XFASTINT (n) != 0)
+      int n = mincol / tab_width - fromcol / tab_width;
+      if (n != 0)
 	{
-	  Finsert_char (make_number ('\t'), n);
+	  Finsert_char (make_number ('\t'), make_number (n));
 
 	  fromcol = (mincol / tab_width) * tab_width;
 	}
     }
 
-  XFASTINT (col) = mincol - fromcol;
-  Finsert_char (make_number (' '), col);
+  Finsert_char (make_number (' '), make_number (mincol - fromcol));
 
   last_known_column_buffer = current_buffer;
   last_known_column = mincol;
@@ -238,8 +245,7 @@ even if that goes past COLUMN; by default, MIN is zero.")
   if (opoint > 0)
     SET_PT (opoint);
 
-  XSETINT (col, mincol);
-  return col;
+  return (make_number (mincol));
 }
 
 int
@@ -272,10 +278,7 @@ This is the horizontal position of the character\n\
 following any initial whitespace.")
   ()
 {
-  Lisp_Object val;
-
-  XFASTINT (val) = position_indentation (find_next_newline (point, -1));
-  return val;
+  return (make_number (position_indentation (find_next_newline (point, -1))));
 }
 
 DEFUN ("move-to-column", Fmove_to_column, Smove_to_column, 1, 2, 0,
@@ -301,7 +304,6 @@ and if COLUMN is in the middle of a tab character, change it to spaces.")
   Lisp_Object ctl_arrow = current_buffer->ctl_arrow;
   register struct Lisp_Vector *dp = buffer_display_table (current_buffer);
 
-  Lisp_Object val;
   int prev_col;
   int c;
 
@@ -353,342 +355,350 @@ and if COLUMN is in the middle of a tab character, change it to spaces.")
   if (!NILP (force) && col > goal && c == '\t' && prev_col < goal)
     {
       del_range (point - 1, point);
-      Findent_to (make_number (col - 1), 0);
+      Findent_to (make_number (col - 1), Qzero);
       insert_char (' ');
       goto retry;
     }
 
   /* If line ends prematurely, add space to the end.  */
   if (col < goal && !NILP (force))
-    Findent_to (make_number (col = goal), 0);
+    {
+      col = goal;
+      Findent_to (make_number (col), Qzero);
+    }
 
   last_known_column_buffer = current_buffer;
   last_known_column = col;
   last_known_column_point = point;
   last_known_column_modified = MODIFF;
 
-  XFASTINT (val) = col;
-  return val;
+  return (make_number (col));
 }
 
 static struct position val_compute_motion;
 
-/* Note that `cpos' is CURRENT_VPOS << SHORTBITS + CURRENT_HPOS,
-   and that CURRENT_HPOS may be negative.  Use these macros
-   to extract the hpos or the vpos from cpos or anything like it. */
-#ifndef SHORT_CAST_BUG
-#define HPOS(VAR) (short) (VAR)
-#else
-#define HPOS(VAR) (((VAR) & (1 << (SHORTBITS - 1)) \
-		    ? ~((1 << SHORTBITS) - 1) : 0) \
-		   | (VAR) & ((1 << SHORTBITS) - 1))
-#endif /* SHORT_CAST_BUG */
-
-#define VPOS(VAR) (((VAR) >> SHORTBITS) + (HPOS (VAR) < 0))
-
-#ifdef HAVE_X_WINDOWS
-#define CHECK_PIX_WIDTH (SCREEN_IS_TERMCAP (screen) || pix_width < max_width)
-#ifdef LINE_INFO_COLUMN
-#define RESET_PIX_WIDTH (pix_width = screen->display.x->line_info_column_width)
-#else
-#define RESET_PIX_WIDTH (pix_width = 0)
-#endif
-#define INC_PIX_WIDTH (pix_width += displayed_glyphs->pixel_width)
-#define PIX_VALUES 1
-#define CHECK_PIX_OVERFLOW (SCREEN_IS_X (screen)			\
-			    && pix_width >= max_width			\
-			    && BUF_CHAR_AT (buffer, pos + 1) != '\n')
-#else  /* not X */
-#define CHECK_PIX_WIDTH 1
-#define RESET_PIX_WIDTH
-#define INC_PIX_WIDTH
-#define PIX_VALUES 0
-#define CHECK_PIX_OVERFLOW 0
-#endif /* not X */
-
-#ifdef LINE_INFO_COLUMN
-#define INFO_COLUMN_ADJUST(cpos) (cpos)++
-#define INFO_COLUMNS(cpos) (HPOS(cpos) - 1)
-#else
-#define INFO_COLUMN_ADJUST(cpos)
-#define INFO_COLUMNS(cpos) (HPOS(cpos))
-#endif
-
-/* Marker for where to display an arrow on top of the buffer text.  */
-Lisp_Object Voverlay_arrow_position;
-
-/* String to display for the arrow.  */
-Lisp_Object Voverlay_arrow_string;
-
-/* New version of compute motion. */
+#define CHECK_NEXT(s) {\
+  a[0] = (s);\
+  wid = text_width (lfont,a,1);\
+  if ((pixpos + wid) > pwidth) {\
+    if (truncate) {\
+      while (bufpos < to && CHAR_AT(bufpos) != '\n') bufpos++;\
+      bufpos--;\
+    } else {\
+      vpos++; hpos = 1; pixpos = wid;\
+      vpixpos += max_line_height;\
+      max_line_height = font->height;\
+      new_line = 1;\
+      tab_offset += window_char_width(w) - 1;\
+    }\
+  } else {\
+    hpos++; pixpos += wid;\
+  }\
+}
 
 struct position *
-compute_motion (window, from, fromvpos, fromhpos, to,
-		tovpos, tohpos, width, hscroll, tab_offset, hpos_pix_width,
-		column)
-     int from, fromvpos, fromhpos, to, tovpos, tohpos, hpos_pix_width, column;
-     register int width;
-     int hscroll, tab_offset;
-     Lisp_Object window;
+compute_motion (struct window *w, int from, int fromvpos, int fromhpos,
+		int to,int tovpos,int tohpos,int hscroll, int tab_offset)
 {
-  struct buffer *buffer = XBUFFER (XWINDOW (window)->buffer);
-  struct Lisp_Vector *dp = buffer_display_table (buffer);
-  SCREEN_PTR screen = XSCREEN (WINDOW_SCREEN (XWINDOW (window)));
-  SCREEN_PTR s = screen;
-  struct window *w = XWINDOW (window);
-#ifdef LINE_INFO_COLUMN
-  int cpos = fromhpos + (fromvpos << SHORTBITS) + 1;
-#else
-  int cpos = fromhpos + (fromvpos << SHORTBITS);
-#endif
-  register int target = tohpos + (tovpos << SHORTBITS);
-  register UCHAR c;
-  register int pos;
-  int prevcpos, pix_width, nextcpos;
-  struct glyphs_from_chars *displayed_glyphs;
-  int overlay_arrow_seen = 0;
+  int hpos = fromhpos;
+  int vpos = fromvpos;
+  int bufpos;
+  int pixpos = 0;
+  int vpixpos, max_line_height, new_line;
+  int prevpos, prevhpos, prevvpos;
+  int savehpos, savevpos, savepos;
+  int c = 0;
+  int c1,wid;
+  unsigned char a[2];
+  int flag = 0;
+  int loop;
 
-#ifdef HAVE_X_WINDOWS
-  int truncate = hscroll
-    || (truncate_partial_width_windows
-	&& XFASTINT (w->width) < SCREEN_WIDTH (s))
-      || !NILP (current_buffer->truncate_lines);
-  register int max_width =
-    (SCREEN_IS_X (screen) ?
-     MAX_LINE_WIDTH (screen) - EOL_CURSOR_WIDTH
-     : 0);
-  /* max_truncated_or_continued_width is widest possible line that is
-     truncated or continued */
-  int max_truncated_or_continued_width =
-        (SCREEN_IS_X (screen) ?
-	 (truncate
-	  ? TRUNCATE_WIDTH (screen)
-	  : CONTINUE_WIDTH (screen))
-	 : max_width);
-  int position_before_continuer = 0;
-  int cpos_before_continuer;
-#endif
+  struct screen *s = XSCREEN (w->screen);
+  struct buffer *b = XBUFFER (w->buffer);
+  int e_start, e_end;
+  struct face *cur_face, *e_face;
+  struct Lisp_Font *font;
+  Lisp_Object lfont;
+  struct glyphs_from_chars *display_info;
 
-  pix_width = hpos_pix_width;
-  pos = from;
-  prevcpos = cpos;
+  int tab_width = XFASTINT (b->tab_width);
+  Lisp_Object ctl_arrow = b->ctl_arrow;
+  int ctl_p = !NILP (ctl_arrow);
+  int printable_min = (FIXNUMP (ctl_arrow)
+		       ? XINT (ctl_arrow)
+		       : ((EQ (ctl_arrow, Qt) || EQ (ctl_arrow, Qnil))
+			  ? 256 : 160));
+  int truncate = hscroll || !NILP(b->truncate_lines)
+    || (truncate_partial_width_windows && w->pixwidth < PIXW(s));
+  int selective = FIXNUMP(b->selective_display)
+    ? XINT (b->selective_display)
+      : !NILP (b->selective_display) ? -1 : 0;
+  int pwidth = w->pixwidth - LEFT_MARGIN (b, s) - RIGHT_MARGIN (b, s);
+  int pheight = (((window_char_height(w) - 1) == tovpos)
+		 ? (w->pixheight
+		    - XFONT (SCREEN_DEFAULT_FONT (s))->height)
+		 : 100000);
+  int char_width;
 
-  while (1)
+  if (tab_width <= 0 || tab_width > 20) tab_width = 8;
+
+  vpixpos = vpos * XFONT (SCREEN_DEFAULT_FONT (s))->height;
+
+  display_info =
+    glyphs_from_bufpos (s,b,from,0,hscroll,0,tab_offset,0,1);
+
+  e_face = display_info->faceptr;
+  e_start = display_info->run_pos_lower;
+  e_end = display_info->run_pos_upper;
+
+  if (e_face)
     {
-      /* There are no glyphs at this position */
-      displayed_glyphs = 0;
-      if (pos == ZV)
+      /* Set font according to the extent's font (if any) */
+      cur_face = e_face;
+      if (!NILP (FACE_FONT (e_face)))
+  	lfont = FACE_FONT (e_face);
+      else
+	lfont = SCREEN_DEFAULT_FONT (s);
+    }
+  else
+    lfont = SCREEN_DEFAULT_FONT (s);
+
+  font = XFONT (lfont);
+  /* Skip any textual glyphs. */
+  for (loop = 0; loop < 2; loop++)
+    {
+      int col_cnt;
+      Lisp_Object *class;
+
+      col_cnt = loop ? display_info->begin_columns
+	: display_info->end_columns;
+      class = loop ? display_info->begin_class
+	: display_info->end_class;
+
+      if (col_cnt)
+	{
+	  int i;
+	  for (i = 0; i < col_cnt; i++)
+	    {
+	      struct extent *e = XEXTENT(class[i]);
+
+	      if (EXTENT_GLYPH_LAYOUT_P (e, GL_TEXT))
+		pixpos += XPIXMAP (glyph_to_pixmap (e->glyph))->width;
+	    }
+	}
+    }
+
+  max_line_height = font->height;
+  new_line = 1;
+
+  if (w == XWINDOW(minibuf_window) && minibuf_prompt_width)
+    pixpos = text_width(lfont, XSTRING(Vminibuf_prompt)->data,
+			XSTRING(Vminibuf_prompt)->size) + 1;
+
+  /* Adjust right logical border of window to account for
+     truncate/continue glyph */
+  pwidth -= (truncate
+	     ? XPIXMAP (glyph_to_pixmap (truncator_glyph))->width
+	     : XPIXMAP (glyph_to_pixmap (continuer_glyph))->width);
+
+  prevpos = from;
+  for (bufpos = from; bufpos < to; bufpos++)
+    {
+      if (vpos > tovpos || (vpos == tovpos && hpos >= tohpos))
 	break;
-
-      /* Deal with the overlay arrow. */
-      if (MARKERP (Voverlay_arrow_position)
-	  && current_buffer == XMARKER (Voverlay_arrow_position)->buffer
-	  && (pos == BEGV
-	      || BUF_CHAR_AT (buffer, pos - 1) == '\n')
-	  && pos == marker_position (Voverlay_arrow_position)
-	  && STRINGP (Voverlay_arrow_string)
-	  && ! overlay_arrow_seen)
+      if (vpixpos + max_line_height > pheight)
 	{
-	  cpos++;
-	  pix_width += builtin_rarrow_pixmap.width;
-	  overlay_arrow_seen = 1;
+	  flag = 1;
 	}
 
-      c = BUF_CHAR_AT (buffer, pos);
-      /* Increment columns by the number of glyphs at this buffer position,
-	 and increment pix_width by the pixel_width of these glyphs. */
-      displayed_glyphs = glyphs_from_bufpos (screen, buffer, pos, dp,
-					     hscroll, column,
-					     tab_offset, 0, 0);
-      nextcpos = cpos + displayed_glyphs->columns;
-      cpos = nextcpos;
-      column += displayed_glyphs->columns - (displayed_glyphs->begin_columns
-						 + displayed_glyphs->end_columns);
-      INC_PIX_WIDTH;
-      
-      /* If we've reached the continuer position,
-	 take note of the pos and cpos before this position. */
+      prevvpos = vpos; prevhpos = hpos;
 
-      if (!position_before_continuer
-	  && pix_width > max_truncated_or_continued_width)
+      if (bufpos == e_start || bufpos == e_end || e_start == -1)
+	{
+	  display_info =
+	    glyphs_from_bufpos (s,b,bufpos,0,hscroll,0,tab_offset,0,1);
+
+	  e_face = display_info->faceptr;
+	  e_start = display_info->run_pos_lower;
+	  e_end = display_info->run_pos_upper;
+
+	  if (e_start == e_end)
+	    e_start = e_end = -1;
+
+	  if (e_face)
 	    {
-	      position_before_continuer = pos - 1;
-	      cpos_before_continuer = prevcpos;
-	    }
-
-	  /* If we've reached the maximum position, go back to the continuer
-	     position and continue on the next line starting with the position
-	     immediately following position_before_continuer */
-	  if (pix_width > max_truncated_or_continued_width)
-	    {
-	      /* Exceeded max_width, go to the next line, starting at
-		 position_before_continuer. */
-	      prevcpos = cpos_before_continuer;
-	      pos = position_before_continuer + 1;
-
-	      if (truncate)
-		{
-		  while (pos < to
-			 && BUF_CHAR_AT (buffer, pos) != '\n')
-		    pos++;
-		  if (BUF_CHAR_AT (buffer, pos) == '\n')
-		    {
-		      displayed_glyphs = 0;
-		      nextcpos = cpos + (1 << SHORTBITS) - HPOS (cpos);
-		      nextcpos -= hscroll;
-		      INFO_COLUMN_ADJUST (nextcpos);
-		      if (hscroll > 0)
-			nextcpos++;	/* Count the ! on column 0 */
-		      tab_offset = 0;
-		      column = 0;
-		      RESET_PIX_WIDTH;
-		      position_before_continuer = 0;
-		    }
-		  if (pos < to || BUF_CHAR_AT (buffer, pos) == '\n')
-		    continue;
-		}
+	      /* Set font according to the extent's font (if any) */
+	      cur_face = e_face;
+	      if (!NILP (FACE_FONT (e_face)))
+		lfont = FACE_FONT (e_face);
 	      else
-		{
-		  cpos += (1 << SHORTBITS) - HPOS (cpos);
-		  cpos -= hscroll;
-		  INFO_COLUMN_ADJUST (cpos);
-		  if (hscroll > 0)
-		    cpos++;	/* Count the ! on column 0 */
-		  tab_offset = 0;
-		  column = 0;
-		  RESET_PIX_WIDTH;
-		  position_before_continuer = 0;
-		  continue;
-		}
+		lfont = SCREEN_DEFAULT_FONT (s);
 	    }
-      if (c == '\n')
-	{
-	  /* If the character is a newline, go to the next display line
-	     positioned at the beginning after any extras like the info
-	     column and the hscroll ! indicator */
-	  displayed_glyphs = 0;	/* Don't adjust cursor for end glyphs on EOL */
-	  nextcpos = cpos + (1 << SHORTBITS) - HPOS (cpos);
-	  nextcpos -= hscroll;
-	  INFO_COLUMN_ADJUST (nextcpos);
+	  else
+	    lfont = SCREEN_DEFAULT_FONT (s);
+	  font = XFONT (lfont);
+	  if (new_line)
+	    max_line_height = font->height;
+	  else
+	    max_line_height = max (font->height, max_line_height);
 
-	  if (hscroll > 0)
-	    nextcpos++;		/* Count the ! on column 0 */
-	  tab_offset = 0;
-	  RESET_PIX_WIDTH;
-	  column = 0;
-	  /* continued_from_position = 0; */
-	  position_before_continuer = 0;
-	}
-
-      if (pos >= to || cpos >= target)
-	{
-	  /* We've found the buffer position we want, but we need to
-	     be sure we aren't in the line wrap zone. */
-	  if (position_before_continuer)
+	  /* Skip any textual glyphs. */
+	  for (loop = 0; loop < 2; loop++)
 	    {
-	      /* If we're in the line wrap zone, we must look ahead to
-		 see if this position is on this line or the next line.
-		 If a line break or end of buffer occurs before max_width,
-		 it's on this line, otherwise it's on the next line. */
-	      int look_c;
-	      int lookahead_pos = pos;
-	      int lookahead_pix_width = pix_width;
-	      struct glyphs_from_chars *lookahead_glyphs;
+	      int col_cnt;
+	      Lisp_Object *class;
 
-	      while (1)
+	      col_cnt = loop ? display_info->begin_columns
+		: display_info->end_columns;
+	      class = loop ? display_info->begin_class
+		: display_info->end_class;
+
+	      if (col_cnt)
 		{
-		  lookahead_pos++;
-		  if (lookahead_pos > ZV)
-		    break;
-		  look_c = BUF_CHAR_AT (buffer, lookahead_pos);
-		  if (look_c == '\n')
-		    /* Found a newline, so we stay on this line */
-		    break;
-		  lookahead_glyphs = glyphs_from_bufpos (screen, buffer,
-							 lookahead_pos, dp,
-							 hscroll,
-							 INFO_COLUMNS (cpos),
-							 tab_offset, 0, 0);
-		  lookahead_pix_width += lookahead_glyphs->pixel_width;
-		  if (lookahead_pix_width > max_truncated_or_continued_width)
+		  int i;
+		  for (i = 0; i < col_cnt; i++)
 		    {
-		      /* Exceeded max_width, go to the next line, starting at
-			 position_before_continuer. */
-		      prevcpos = cpos_before_continuer;
-		      pos = position_before_continuer + 1;
+		      struct extent *e = XEXTENT(class[i]);
 
-		      if (truncate)
-			{
-			  while (pos < to
-				 && BUF_CHAR_AT (buffer, pos) != '\n')
-			    pos++;
-			  if (pos >= to)
-			    cpos = cpos_before_continuer + 2;
-			  /* Prevent adjustment after breaking */
-			  displayed_glyphs = 0;
-			}
-		      else
-			{
-			  cpos += (1 << SHORTBITS) - HPOS (cpos);
-			  cpos -= hscroll;
-			  INFO_COLUMN_ADJUST (cpos);
-			  if (hscroll > 0)
-			    cpos++; /* Count the ! on column 0 */
-			  tab_offset = 0;
-			  RESET_PIX_WIDTH;
-			  column = 0;
-			  position_before_continuer = 0;
-			}
-		      break;
+		      if (EXTENT_GLYPH_LAYOUT_P (e, GL_TEXT))
+			pixpos += XPIXMAP (glyph_to_pixmap (e->glyph))->width;
 		    }
 		}
-	      if (pos >= to)
-		break;
+	    }
+	}
+
+      c = CHAR_AT(bufpos);
+      a[0] = c;
+      char_width = text_width (lfont, a, 1);
+
+      if (c >= printable_min && char_width)
+	{
+	  CHECK_NEXT(c);
+	}
+      else if (c == '\t')
+	{
+	  int i;
+	  int inc = tab_width -
+	    ((hpos + tab_offset + hscroll - (hscroll > 0)
+	      /* Add tab_width here to make sure positive.
+		 hpos can be negative after continuation but
+		 can't be less than -tab_width. */
+	      + tab_width)
+	      % tab_width);
+	  for (i = 0; i < inc; i++)
+	    CHECK_NEXT(' ');
+	}
+      else if (c == '\n')
+	{
+	  if (selective > 0 && position_indentation(bufpos + 1) >= selective)
+	    {
+	      /* Skip invisible lines */
+	      do
+		{
+		  while (++bufpos < to && CHAR_AT(bufpos) != '\n');
+		}
+	      while (selective > 0
+		     && position_indentation(bufpos + 1) >= selective);
+	      bufpos--;
+	      /* Allow for ' ...' */
+	      if (!NILP(current_buffer->selective_display_ellipses))
+		{
+		  int i;
+		  CHECK_NEXT(' ');
+		  for (i = 0; i < 3; i++)
+		    CHECK_NEXT('.');
+		}
 	    }
 	  else
 	    {
-	      /* We aren't in the line wrap zone, so we're done.
-		 prevpos points to the previous position. */
-	      break;
+	      vpos++;
+	      prevpos = bufpos;
+	      vpixpos += max_line_height;
+	      max_line_height = font->height;
+	      new_line = 1;
+	      pixpos = hpos = 0;
 	    }
+	  hpos -= hscroll;
+	  if (hscroll > 0)
+	    {
+	      CHECK_NEXT('$');
+	    }
+	  tab_offset = 0;
+	}
+      else if (c == CR && selective < 0)
+	{
+	  while (bufpos < to && CHAR_AT(bufpos) != '\n') bufpos++;
+	  bufpos--;
+	  /* Allow for ' ...' */
+	  if (!NILP(current_buffer->selective_display_ellipses))
+	    {
+	      int i;
+	      CHECK_NEXT(' ');
+	      for (i = 0; i < 3; i++)
+		CHECK_NEXT('.');
+	    }
+	}
+      else if (char_width && ((c < 040 && ctl_p) || c == 0177))
+	{
+	  /* Insert control character as a 2-char sequence */
+	  CHECK_NEXT('^');
+	  if (c == 0177)
+	    {
+	      CHECK_NEXT ('?');
+	    }
+	  else
+	    {
+	      CHECK_NEXT(c ^ 0100);
+	    }
+	}
+      else if (c >= 0200 || c < 040 || !char_width)
+	{
+	  /* Insert control character as a 4-char sequence */
+	  CHECK_NEXT('\\');
+	  CHECK_NEXT((c>>6) + '0');
+	  CHECK_NEXT((7 & (c>>3)) + '0');
+	  CHECK_NEXT((7 & c) + '0');
 	}
       else
 	{
-	  prevcpos = (displayed_glyphs
-		      ? (cpos - (displayed_glyphs->columns
-				 - displayed_glyphs->begin_columns))
-		      : cpos);
-	  cpos = nextcpos;
-	  pos++;
+	  CHECK_NEXT(c);
 	}
+      new_line = 0;
     }
 
-  /* cpos now points to the character following this position, so we must
-     subtract all but the begin glyphs to get this position.  If this
-     position is at a newline or in the truncation zone, there are no glyphs
-     and cpos points to this position and needs no adjustment. */
-  if (displayed_glyphs)
+  savevpos = vpos;
+  savehpos = hpos;
+  savepos = bufpos;
+
+  /* If at end of buffer/narrowed region, don't fetch next character as
+   * it isn't there.  Fake it with last character of buffer.
+   */
+  c1 = ((bufpos < ZV) ? CHAR_AT(bufpos) : c);
+  if (c1)
+    CHECK_NEXT(c1);
+  if (vpos != savevpos && savepos < ZV && (c1 != '\n') && !truncate)
     {
-      cpos -= (displayed_glyphs->columns
-	       - displayed_glyphs->begin_columns);
-      pix_width -= displayed_glyphs->pixel_width;
-      column -= (displayed_glyphs->columns
-		 - (displayed_glyphs->begin_columns + displayed_glyphs->end_columns));
+      vpos = savevpos + 1; hpos = 0;
     }
+  else
+    {
+      vpos = savevpos; hpos = savehpos;
+    }
+  bufpos = savepos;
 
-  val_compute_motion.bufpos = pos;
-  val_compute_motion.hpos = HPOS (cpos);
-  val_compute_motion.vpos = VPOS (cpos);
-  val_compute_motion.prevhpos = HPOS (prevcpos);
-  val_compute_motion.pixpos = pix_width;
-  val_compute_motion.column = column;
+  val_compute_motion.bufpos = bufpos;
+  val_compute_motion.hpos = hpos;
+  val_compute_motion.vpos = vpos;
+  val_compute_motion.prevhpos = prevhpos;
 
-  /* Nonzero if have just continued a line */
-  val_compute_motion.contin = ((pos != from
-				&& (val_compute_motion.vpos != VPOS (prevcpos))
-				&& c != '\n')
-			       ? (pos - 1)
-			       : 0);
+  val_compute_motion.contin
+    = bufpos != from && (val_compute_motion.vpos != prevvpos) && c != '\n';
+
+  if (flag)
+    {
+      val_compute_motion.bufpos = prevpos;
+/*      vpos--; hpos = 0; bufpos = prevpos; */
+    }
 
   return &val_compute_motion;
 }
@@ -696,31 +706,8 @@ compute_motion (window, from, fromvpos, fromhpos, to,
 
 #undef HPOS
 #undef VPOS
+#undef CHECK_NEXT
 
-int pos_tab_offset (struct window *, int);
-
-DEFUN ("motion", Fmotion, Smotion, 3, 3, 0,
-  "Move forward from point by N characters.  Stop if we reach\n\
-TOHPOS, TOVPOS first.")
-  (n, tohpos, tovpos)
-     Lisp_Object n, tohpos, tovpos;
-{
-  int fromhpos = 0;
-  int fromvpos = SCREEN_CURSOR_Y (selected_screen);
-  int width = XFASTINT (XWINDOW (selected_window)->width);
-  int hscroll = XINT (XWINDOW (selected_window)->hscroll);
-  struct position *pos;
-
-  pos = compute_motion (selected_window,
-			point, fromvpos, fromhpos, (point + n),
-			XINT (tovpos), XINT (tohpos),
-			width, hscroll,
-			pos_tab_offset (XWINDOW (selected_window), point),
-			0, 0);
-  SET_PT (pos->bufpos);
-  return Fcons (make_number (pos->vpos),
-		Fcons (make_number (pos->hpos), Qnil));
-}
 
 /* Return the column of position POS in window W's buffer,
    rounded down to a multiple of the internal width of W.
@@ -751,6 +738,30 @@ pos_tab_offset (w, pos)
 #endif
 }
 
+DEFUN ("motion", Fmotion, Smotion, 3, 3, 0,
+  "Move forward from point by N characters.  Stop if we reach\n\
+TOHPOS, TOVPOS first.")
+  (n, tohpos, tovpos)
+     Lisp_Object n, tohpos, tovpos;
+{
+  int fromhpos = 0;
+  int fromvpos = SCREEN_CURSOR_Y (selected_screen);
+  int hscroll = XINT (XWINDOW (selected_window)->hscroll);
+  struct position *pos;
+
+  CHECK_FIXNUM (n, 0);
+  CHECK_FIXNUM (tohpos, 1);
+  CHECK_FIXNUM (tovpos, 2);
+
+  pos = compute_motion (XWINDOW (selected_window),
+			point, fromvpos, fromhpos, (point + XINT (n)),
+			XINT (tovpos), XINT (tohpos), hscroll,
+			pos_tab_offset (XWINDOW (selected_window), point));
+  SET_PT (pos->bufpos);
+  return Fcons (make_number (pos->vpos),
+		Fcons (make_number (pos->hpos), Qnil));
+}
+
 /* start_hpos is the hpos of the first character of the buffer:
    zero except for the minibuffer window,
    where it is the width of the prompt.  */
@@ -758,14 +769,15 @@ pos_tab_offset (w, pos)
 static struct position val_vmotion;
 
 struct position *
-vmotion (from, vtarget, width, hscroll, window)
-     register int from, vtarget, width;
+vmotion (from, vtarget, hscroll, window)
+     register int from, vtarget;
      int hscroll;
      Lisp_Object window;
 {
   struct position pos;
   /* vpos is cumulative vertical position, changed as from is changed */
   register int vpos = 0;
+  register int width = window_char_width (XWINDOW (window));
   register int prevline;
   register int first;
   int lmargin = hscroll > 0 ? 1 - hscroll : 0;
@@ -774,8 +786,6 @@ vmotion (from, vtarget, width, hscroll, window)
       ? XINT (current_buffer->selective_display)
 	: !NILP (current_buffer->selective_display) ? -1 : 0;
   int start_hpos = (EQ (window, minibuf_window) ? minibuf_prompt_width : 0);
-  int start_pixpos = (EQ (window, minibuf_window) ? minibuf_prompt_pix_width : 0);
-  
 
  retry:
   if (vtarget > vpos)
@@ -789,24 +799,21 @@ vmotion (from, vtarget, width, hscroll, window)
 		 && prevline > BEGV
 		 && position_indentation (prevline) >= selective)
 	    prevline = find_next_newline (prevline - 1, -1);
-	  pos = *compute_motion (window,
+	  pos = *compute_motion (XWINDOW (window),
 				 prevline, 0,
 				 lmargin + (prevline == 1 ? start_hpos : 0),
 				 from, 10000, 10000,
-				 width, hscroll, 0, start_pixpos, 0);
+				 hscroll, 0);
 	}
       else
 	{
 	  pos.hpos = lmargin + (from == 1 ? start_hpos : 0);
-	  pos.pixpos = (from == 1 ? start_pixpos : 0);
-	  pos.column = 0;
 	  pos.vpos = 0;
 	}
-      return compute_motion (window,
+      return compute_motion (XWINDOW (window),
 			     from, vpos, pos.hpos,
 			     ZV, vtarget, - (1 << (SHORTBITS - 1)),
-			     width, hscroll, pos.vpos * width,
-			     pos.pixpos, pos.column);
+			     hscroll, (pos.vpos * width));
     }
 
   /* To move upward, go a line at a time until
@@ -825,11 +832,11 @@ vmotion (from, vtarget, width, hscroll, window)
 	      || position_indentation (prevline) < selective)
 	    break;
 	}
-      pos = *compute_motion (window,
+      pos = *compute_motion (XWINDOW (window),
 			     prevline, 0,
 			     lmargin + (prevline == 1 ? start_hpos : 0),
 			     from, 10000, 10000,
-			     width, hscroll, 0, start_pixpos, 0);
+			     hscroll, 0);
       vpos -= pos.vpos;
       first = 0;
       from = prevline;
@@ -843,8 +850,6 @@ vmotion (from, vtarget, width, hscroll, window)
       val_vmotion.bufpos = from;
       val_vmotion.vpos = vpos;
       val_vmotion.hpos = lmargin;
-      val_vmotion.pixpos = 0;
-      val_vmotion.column = 0;
       val_vmotion.contin = 0;
       val_vmotion.prevhpos = 0;
       return &val_vmotion;
@@ -870,19 +875,16 @@ Optional second argument is WINDOW to move in.")
   {
     struct position pos;
     register struct window *w  = XWINDOW (window);
-    int width = XFASTINT (w->width) - 1
-      - (XFASTINT (w->width) + XFASTINT (w->left)
-	 != SCREEN_WIDTH (XSCREEN (w->screen)));
 
     CHECK_FIXNUM (lines, 0);
 
-    pos = *vmotion (point, XINT (lines), width,
-		    XINT (w->hscroll), window);
+    pos = *vmotion (point, XINT (lines), XINT (w->hscroll), window);
 
     SET_PT (pos.bufpos);
     return make_number (pos.vpos);
   }
 }
+
 
 void
 syms_of_indent ()
